<?php

namespace Drupal\aws_cloud\Plugin\Block;

use Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface;
use Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Link;

/**
 * Provides a block displaying snapshot information.
 *
 * @Block(
 *   id = "aws_cloud_snapshots_block",
 *   admin_label = @Translation("Snapshot Blocks"),
 *   category = @Translation("AWS Cloud")
 * )
 */
class SnapshotsBlock extends BlockBase implements ContainerFactoryPluginInterface {

  use StringTranslationTrait;

  /**
   * Stores the configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Ec2 Service interface.
   *
   * @var \Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface
   */
  protected $ec2Service;

  /**
   * The cloud service provider plugin manager (CloudConfigPluginManager).
   *
   * @var \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface
   */
  protected $cloudConfigPluginManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Creates a StaleSnapshotsBlock instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\aws_cloud\Service\Ec2\Ec2ServiceInterface $ec2Service
   *   The Ec2 service interface.
   * @param \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface $cloud_config_plugin_manager
   *   The cloud service provider plugin manager (CloudConfigPluginManager).
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ConfigFactoryInterface $config_factory,
    AccountInterface $current_user,
    Ec2ServiceInterface $ec2Service,
    CloudConfigPluginManagerInterface $cloud_config_plugin_manager,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
    $this->ec2Service = $ec2Service;
    $this->cloudConfigPluginManager = $cloud_config_plugin_manager;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('current_user'),
      $container->get('aws_cloud.ec2'),
      $container->get('plugin.manager.cloud_config_plugin'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'snapshot_block_type' => 'orphaned_snapshots',
      'cloud_context' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $unused_days = $this->configFactory->get('aws_cloud.settings')->get('aws_cloud_stale_snapshot_criteria');
    $form['cloud_context'] = [
      '#type' => 'select',
      '#title' => $this->t('Cloud Service Provider'),
      '#description' => $this->t('Select cloud service provider'),
      '#options' => $this->getCloudConfigs(),
      '#default_value' => $this->configuration['cloud_context'] ?? '',
    ];

    $form['snapshot_block_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Snapshot Blocks to Show'),
      '#options' => [
        'orphaned_snapshots' => $this->t(
          'Show the snapshots orphaned from AMI Images.'
        ),
        'disassociated_snapshots' => $this->t(
          'Show the snapshots disassociated from volumes.'
        ),
        'stale_snapshots' => $this->t(
          'Show snapshots created more than %d days ago',
          [
            '%d' => Link::createFromRoute($unused_days, 'aws_cloud.settings.options')->toString(),
          ]
        ),
      ],
      '#description' => $this->t('Choose the Snapshot information to display'),
      '#default_value' => $this->configuration['snapshot_block_type'],
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['snapshot_block_type']
      = $form_state->getValue('snapshot_block_type');
    $this->configuration['cloud_context']
      = $form_state->getValue('cloud_context');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    $type = $this->configuration['snapshot_block_type'];

    if ($type === 'orphaned_snapshots') {
      $build[] = $this->buildOrphanedSnapshots();
    }
    elseif ($type === 'disassociated_snapshots') {
      $build[] = $this->buildDisassociatedSnapshots();
    }
    elseif ($type === 'stale_snapshots') {
      $build[] = $this->buildStaleSnapshotList();
    }

    return $build;
  }

  /**
   * Build a list of stale snapshots.
   *
   * @return array
   *   A set of render arrays.
   */
  private function buildStaleSnapshotList() {

    $build['snapshots'] = [
      '#type' => 'details',
      '#title' => $this->t('Stale Snapshots'),
      '#open' => TRUE,
    ];
    $snapshots = $this->getStaleSnapshots();
    if (count($snapshots)) {
      $unused_days = $this->configFactory->get('aws_cloud.settings')->get('aws_cloud_stale_snapshot_criteria');
      $urls = [];
      /* @var \Drupal\aws_cloud\Entity\Ec2\Snapshot $snapshot */
      foreach ($snapshots ?: [] as $snapshot) {
        try {
          $days_running = $snapshot->daysRunning();
          $link_text = $this->t('@snapshot (%running @days)', [
            '@snapshot' => $snapshot->getName(),
            '%running' => $days_running,
            '@days' => $this->formatPlural($days_running, 'day', 'days'),
          ]);
          $urls[] = $snapshot->toLink($link_text);
        }
        catch (\Exception $e) {
          $this->messenger()->addError("An error occurred: {$e->toString()}");
        }
      }
      $build['snapshots'][] = [
        '#markup' => $this->t('The following snapshots have been running for more than %num days', ['%num' => $unused_days]),
      ];
      $build['snapshots'][] = [
        '#theme' => 'item_list',
        '#items' => $urls,
      ];
    }
    else {
      $build['snapshots'] = [
        '#markup' => $this->t('Great job! You have no stale snapshots.'),
      ];
    }
    return $build;
  }

  /**
   * Get a list of stale snapshots.
   *
   * @return array
   *   Array of stale snapshots.
   */
  private function getStaleSnapshots() {
    $snapshots = aws_cloud_get_stale_snapshots($this->configuration['cloud_context']);
    if (!$this->currentUser->hasPermission('view any aws cloud snapshot')) {
      /* @var \Drupal\aws_cloud\Entity\Ec2\Snapshot $snapshot */
      foreach ($snapshots ?: [] as $key => $snapshot) {
        // Only return volumes the user has access to.
        if ($snapshot->getOwnerId() !== $this->currentUser->id()) {
          unset($snapshots[$key]);
        }
      }
    }
    return $snapshots;
  }

  /**
   * Build a list of orphaned snapshots.
   *
   * @return array
   *   An array of urls linking to the snapshot in question.
   */
  private function buildOrphanedSnapshots() {
    $build = [];

    $cloud_context = $this->configuration['cloud_context'] ?? '';

    $image_snapshot_ids = $this->getImagesWithSnapshotIds($cloud_context);
    if (!empty($image_snapshot_ids)) {

      $build['orphaned_snapshots'] = [
        '#type' => 'details',
        '#title' => $this->t('Orphaned Snapshots'),
        '#open' => TRUE,
      ];

      $snapshots = $this->getOrphanedSnapshots($image_snapshot_ids, $cloud_context);
      if (!empty($snapshots)) {
        $urls = [];
        foreach ($snapshots ?: [] as $snapshot) {
          $days_running = $snapshot->daysRunning();
          $link_text = $this->t('@snapshot (%unused @days)', [
            '@snapshot' => $snapshot->getName(),
            '%unused' => $days_running,
            '@days' => $this->formatPlural($snapshot->daysRunning(), 'day', 'days'),
          ]);
          $urls[] = $snapshot->toLink($link_text);
        }
        $build['orphaned_snapshots'][] = [
          '#markup' => $this->t('The following snapshots are orphaned'),
        ];
        $build['orphaned_snapshots'][] = [
          '#theme' => 'item_list',
          '#items' => $urls,
        ];
      }
      else {
        $build['orphaned_snapshots'] = [
          '#markup' => $this->t('Great job!  You have no orphaned snapshots'),
        ];
      }
    }
    else {
      $build['orphaned_snapshots'] = [
        '#markup' => $this->t('Great job!  You have no orphaned snapshots'),
      ];
    }
    return $build;
  }

  /**
   * Helper method to retrieve Images associated with a snapshot.
   *
   * @param string $cloud_context
   *   Cloud context to query.
   *
   * @return array|\Drupal\Core\Entity\EntityInterface[]
   *   An array snapshot_ids
   */
  private function getImagesWithSnapshotIds($cloud_context = '') {
    $image_snapshot_ids = [];
    try {
      $query = $this->entityTypeManager
        ->getStorage('aws_cloud_image')
        ->getQuery()
        ->condition('status', 'available');
      if (!empty($cloud_context)) {
        $query->condition('cloud_context', $cloud_context);
      }
      $and = $query->andConditionGroup();
      $and->exists('block_device_mappings.snapshot_id');

      $ids = $query->condition($and)->execute();
      if (!empty($ids)) {
        $images = $this->entityTypeManager
          ->getStorage('aws_cloud_image')
          ->loadMultiple($ids);

        // Loop through the image block_device_mappings field and
        // find any snapshot_ids.
        foreach ($images ?: [] as $image) {
          $block_device_mappings = $image->getBlockDeviceMappings();
          foreach ($block_device_mappings ?: [] as $block_device) {
            $snapshot_id = $block_device->getSnapshotId();
            if (!empty($snapshot_id)) {
              $image_snapshot_ids[] = $block_device->getSnapshotId();
            }
          }
        }
      }
    }
    catch (\Exception $e) {
      $this->messenger()->addError("An error occurred: {$e->toString()}");
    }
    return $image_snapshot_ids;
  }

  /**
   * Query for a list of orphaned snapshots.
   *
   * The query takes a list of ami images that contains snapshot_ids.  It
   * searches the aws_cloud_snapshot table to find any snapshots that do
   * not match the snapshot_ids from the $images array.
   *
   * @param array $image_snapshot_ids
   *   An array of snapshot_ids.
   * @param string $cloud_context
   *   Cloud context to query.
   *
   * @return array|\Drupal\Core\Entity\EntityInterface[]
   *   An array of snapshots.
   */
  private function getOrphanedSnapshots(array $image_snapshot_ids, $cloud_context = '') {
    $snapshots = [];

    try {
      if (!empty($image_snapshot_ids)) {
        $entity_id = 'aws_cloud_snapshot';
        $query = $this->entityTypeManager
          ->getStorage($entity_id)
          ->getQuery()
          ->condition('snapshot_id', $image_snapshot_ids, 'NOT IN');

        if (!empty($cloud_context)) {
          $query->condition('cloud_context', $cloud_context);
        }
        $ids = $query->execute();
        if (!empty($ids)) {
          $snapshots = $this->entityTypeManager
            ->getStorage($entity_id)
            ->loadMultiple($ids);
        }
      }
    }
    catch (\Exception $e) {
      $this->messenger()->addError("An error occurred: {$e->toString()}");
    }
    return $snapshots;
  }

  /**
   * Build a list of unused snapshots.
   *
   * @return array
   *   A set of render arrays.
   */
  private function buildDisassociatedSnapshots() {
    $build = [];
    $build['snapshot'] = [
      '#type' => 'details',
      '#title' => $this->t('Disassociated Snapshots'),
      '#open' => TRUE,
    ];

    $snapshots = $this->getDisassociatedSnapshots();
    if (count($snapshots)) {
      $urls = [];
      /* @var \Drupal\aws_cloud\Entity\Ec2\Snapshot $snapshot */
      foreach ($snapshots ?: [] as $snapshot) {
        try {
          $urls[] = $snapshot->toLink($snapshot->getName());
        }
        catch (\Exception $e) {
          $this->messenger()->addError("An error occurred: {$e->toString()}");
        }
      }
      $build['snapshot'][] = [
        '#markup' => $this->t('The following snapshots are not associated with a volume'),
      ];
      $build['snapshot'][] = [
        '#theme' => 'item_list',
        '#items' => $urls,
      ];
    }
    else {
      $build['snapshot'] = [
        '#markup' => $this->t('Great job! You have no disassociated snapshots.'),
      ];
    }
    return $build;
  }

  /**
   * Get a list of disassociated snapshots.
   *
   * @return array
   *   Array of disassociated snapshots.
   */
  private function getDisassociatedSnapshots() {
    $snapshots = aws_cloud_get_unused_snapshots($this->configuration['cloud_context']);
    if (!$this->currentUser->hasPermission('view any aws cloud snapshot')) {
      /* @var \Drupal\aws_cloud\Entity\Ec2\Snapshot $snapshot */
      foreach ($snapshots ?: [] as $key => $snapshot) {
        // Only return volumes the user has access to.
        if ($snapshot->getOwnerId() !== $this->currentUser->id()) {
          unset($snapshots[$key]);
        }
      }
    }
    return $snapshots;
  }

  /**
   * Load the cloud configs as an array for use in a select dropdown.
   *
   * @return array
   *   An array of cloud configs.
   */
  private function getCloudConfigs() {
    $cloud_configs = ['' => $this->t('All AWS Cloud regions')];
    $configs = $this->cloudConfigPluginManager->loadConfigEntities('aws_cloud');
    foreach ($configs ?: [] as $config) {
      $cloud_configs[$config->getCloudContext()] = $config->getName();
    }
    return $cloud_configs;
  }

}
