<?php

namespace Drupal\Tests\aws_cloud\Functional\Config;

use Drupal\Component\Utility\Random;
use Drupal\Core\Link;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\Tests\aws_cloud\Traits\AwsCloudTestEntityTrait;
use Drupal\Tests\aws_cloud\Traits\AwsCloudTestFormDataTrait;
use Drupal\Tests\BrowserTestBase;
use Drupal\user\Entity\Role;
use Drupal\user\RoleInterface;

/**
 * Tests cloud service provider (CloudConfig).
 *
 * @group AWS Cloud
 */
class AwsCloudConfigTest extends BrowserTestBase {

  use StringTranslationTrait;
  use AwsCloudTestEntityTrait;
  use AwsCloudTestFormDataTrait;

  public const AWS_CLOUD_CONFIG_REPEAT_COUNT = 1;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = [
    'cloud',
    'aws_cloud',
  ];

  /**
   * The profile to install as a basis for testing.
   *
   * @var string
   */
  protected $profile = 'minimal';

  /**
   * The random string.
   *
   * @var string
   */
  protected $random;

  /**
   * Set up test.
   */
  protected function setUp() {

    parent::setUp();

    if (!$this->random) {
      $this->random = new Random();
    }

    $config = \Drupal::configFactory()->getEditable('aws_cloud.settings');
    $config->set('aws_cloud_test_mode', TRUE)
      ->save();

    $this->initMockInstanceTypes();

    $web_user = $this->drupalCreateUser([
      'administer cloud service providers',
      'add cloud service providers',
      'edit cloud service providers',
      'edit own cloud service providers',
      'delete cloud service providers',
      'delete own cloud service providers',
      'view unpublished cloud service providers',
      'view own unpublished cloud service providers',
      'view published cloud service providers',
      'view own published cloud service providers',
      'access dashboard',
      'view cloud service provider admin list',
      'list cloud server template',
      'administer aws_cloud',
    ]);

    $this->drupalLogin($web_user);
  }

  /**
   * Tests CRUD for cloud service provider (CloudConfig) information.
   */
  public function testCloudConfig() {

    // Get all regions.
    $regions = \Drupal::service('aws_cloud.ec2')->getRegions();
    $target_region_index = rand(0, count($regions) - 1);
    $target_region_display_name = $regions[array_keys($regions)[$target_region_index]];

    // List AWS Cloud for Amazon EC2.
    $this->drupalGet('/admin/structure/cloud_config');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextNotContains($this->t('Notice'));
    $this->assertSession()->pageTextNotContains($this->t('Warning'));

    // Add a new Config information.
    $add = $this->createCloudConfigTestFormData();
    for ($i = 0; $i < self::AWS_CLOUD_CONFIG_REPEAT_COUNT; $i++) {
      // We need to specify the 'view <CLOUD_CONTEXT>' permission.
      $label = "{$add[$i]['name[0][value]']} {$target_region_display_name}";
      $region_index = 0;
      foreach ($regions ?: [] as $region_name => $region_display_name) {
        if ($region_index === $target_region_index) {
          $add[$i]["regions[$region_name]"] = $region_name;
        }
        else {
          $add[$i]["regions[$region_name]"] = '';
        }
        $this->grantPermissions(
          Role::load(
            RoleInterface::AUTHENTICATED_ID),
            [
              'view ' . aws_cloud_form_cloud_config_aws_cloud_add_form_create_cloud_context(
                $add[$i]['name[0][value]'],
                $region_name),
            ]
        );

        $region_index++;
      }

      // These variable that are sent from a browser are specified in the form
      // in the browser, therefore we don't use the auto-generated test values
      // by createCloudConfigTestFormData.
      unset($add[$i]['field_region']);
      unset($add[$i]['field_api_endpoint_uri']);
      unset($add[$i]['field_image_upload_url']);

      $this->drupalGet('/admin/structure/cloud_config/add');
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));

      $this->drupalPostForm('/admin/structure/cloud_config/add',
        $add[$i],
        $this->t('Save'));

      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
      $this->assertSession()->pageTextContains($this->t('Creating cloud service provider was performed successfully.'));

      $this->assertSession()->pageTextContains($label);

      // Make sure listing for '/admin/structure/cloud_config'.
      $this->drupalGet('/admin/structure/cloud_config');
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
      $this->assertSession()->linkExists($label);

      // Make sure listing for '/clouds'.
      $this->drupalGet('/clouds');
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
      $this->assertSession()->linkExists($label);

      // Make sure listing for '/admin/config/services/cloud/aws_cloud'.
      $this->drupalGet('/admin/config/services/cloud/aws_cloud');
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
      $this->assertSession()->linkExists($label);
    }

    // Edit Config case.
    $labels = [];
    $edit = $this->createCloudConfigTestFormData();
    $cloud_config_prefix = $edit[0]['name[0][value]'];
    for ($i = 0; $i < self::AWS_CLOUD_CONFIG_REPEAT_COUNT; $i++) {
      $num = 1;
      $label = "${cloud_config_prefix} ${target_region_display_name}";
      $edit[$i]['name[0][value]'] = $label;

      $this->drupalPostForm("/admin/structure/cloud_config/${num}/edit",
        $edit[$i],
        $this->t('Save'));
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));

      $this->assertSession()->pageTextContains($label);

      // Make sure listing for '/admin/structure/cloud_config'.
      $this->drupalGet('/admin/structure/cloud_config');
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
      $this->assertSession()->linkExists($label);

      // Make sure listing for '/clouds'.
      $this->drupalGet('/clouds');
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
      $this->assertSession()->linkExists($label);

      // Make sure listing for '/admin/config/services/cloud/aws_cloud'.
      $this->drupalGet('/admin/config/services/cloud/aws_cloud');
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
      $this->assertSession()->linkExists($label);
    }

    // Delete Config Items.
    for ($i = 0; $i < self::AWS_CLOUD_CONFIG_REPEAT_COUNT; $i++) {
      $num = 1;
      $this->drupalGet("/admin/structure/cloud_config/${num}/delete");
      $this->drupalPostForm("/admin/structure/cloud_config/${num}/delete",
        [],
        $this->t('Delete'));
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($this->t('Notice'));
      $this->assertSession()->pageTextNotContains($this->t('Warning'));
    }
  }

  /**
   * Tests deleting cloud service provider (CloudConfig) with bulk operation.
   */
  public function testCloudConfigBulk() {

    for ($i = 0; $i < self::AWS_CLOUD_CONFIG_REPEAT_COUNT; $i++) {
      // Create cloud service providers.
      $cloud_configs = $this->createCloudConfigRandomTestFormData();
      $index = 0;
      $entities = [];
      foreach ($cloud_configs ?: [] as $cloud_config) {
        $entity = $this->createCloudConfigTestEntity($index++, $cloud_config['CloudContext'], $cloud_config['Name']);

        $this->grantPermissions(
          Role::load(
            RoleInterface::AUTHENTICATED_ID),
          ["view {$entity->getCloudContext()}"]
        );
        $entities[] = $entity;
      }

      $this->drupalGet('/admin/structure/cloud_config');
      $this->doTestEntityBulk(
        'cloud_config',
        $entities,
        'delete',
        'deleted',
        '/admin/structure'
      );

      $this->drupalGet('/admin/structure/cloud_config');
      foreach ($cloud_configs ?: [] as $cloud_config) {
        $this->assertSession()->linkNotExists($cloud_config['Name']);
      }

      $this->drupalGet('/clouds');
      foreach ($cloud_configs ?: [] as $cloud_config) {
        $this->assertSession()->linkNotExists($cloud_config['Name']);
      }
    }
  }

  /**
   * Tests Redirect for cloud service provider (CloudConfig) information.
   */
  public function testCloudConfigRedirect() {
    $this->repeatTestCloudConfigRedirect(self::AWS_CLOUD_CONFIG_REPEAT_COUNT);
  }

  /**
   * Repeating test cloud service provider redirect.
   *
   * @param int $max_count
   *   Max test repeating count.
   */
  private function repeatTestCloudConfigRedirect($max_count) {
    $paths = [
      '/clouds',
      '/clouds/design',
    ];

    $url = Url::fromRoute('entity.cloud_config.add_page');
    $page_link = Link::fromTextAndUrl(t('cloud service provider'), $url)
      ->toString();
    $message = t('There is no cloud service provider. Please create a new @link.', [
      '@link' => $page_link,
    ]);

    foreach ($paths ?: [] as $path) {
      for ($i = 0; $i < $max_count; $i++) {
        $this->drupalGet($path);
        $this->assertSession()->statusCodeEquals(200);
        $this->assertSession()->pageTextNotContains($this->t('Notice'));
        $this->assertSession()->pageTextNotContains($this->t('Warning'));

        $this->assertSession()->pageTextContains($this->t('Add cloud service provider'));
        $this->assertRaw(
          $this->t('There is no cloud service provider. Please create a new @link.', [
            '@link' => $page_link,
          ]),
          $this->t('Guid message')
        );
      }
    }
  }

}
