<?php

namespace Drupal\Tests\aws_cloud\Functional\Vpc;

use Drupal\Tests\aws_cloud\Functional\AwsCloudTestCase;

/**
 * Tests AWS Cloud Subnet.
 *
 * @group AWS Cloud
 */
class SubnetTest extends AwsCloudTestCase {

  public const AWS_CLOUD_SUBNET_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'list aws cloud subnet',
      'add aws cloud subnet',
      'view any aws cloud subnet',
      'edit any aws cloud subnet',
      'delete any aws cloud subnet',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function getMockDataTemplateVars() {
    return [
      'subnet_id' => 'subnet-' . $this->getRandomAwsId(),
    ];
  }

  /**
   * Tests CRUD for Subnet information.
   */
  public function testSubnet() {
    $cloud_context = $this->cloudContext;

    // List Subnet for Amazon EC2.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/subnet");
    $this->assertSession()->statusCodeEquals(200);

    $vpc_ids = $this->updateVpcsMockData(self::AWS_CLOUD_SUBNET_REPEAT_COUNT);

    // Add a new Subnet.
    $add = $this->createSubnetTestFormData(self::AWS_CLOUD_SUBNET_REPEAT_COUNT, $vpc_ids);
    for ($i = 0; $i < self::AWS_CLOUD_SUBNET_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $subnet_id = $this->latestTemplateVars['subnet_id'];
      $this->addSubnetMockData($add[$i], $subnet_id);

      $num = $i + 1;

      $this->drupalPostForm(
        "/clouds/aws_cloud/$cloud_context/subnet/add",
        $add[$i],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('AWS Cloud Subnet @name has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/subnet");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($add[$i]['name']);

      $add[$i]['subnet_id'] = $subnet_id;
    }

    // Edit a Subnet.
    $edit = $this->createSubnetTestFormData(self::AWS_CLOUD_SUBNET_REPEAT_COUNT, $vpc_ids);
    for ($i = 0; $i < self::AWS_CLOUD_SUBNET_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      unset($edit[$i]['vpc_id']);
      unset($edit[$i]['cidr_block']);

      $this->modifySubnetMockData($i, $edit[$i]);

      $this->drupalPostForm(
        "/clouds/aws_cloud/$cloud_context/subnet/$num/edit",
        $edit[$i],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains(t(
        'AWS Cloud Subnet @name has been updated.',
        ['@name' => $edit[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/subnet");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($edit[$i]['name']);
    }

    // Update tags for empty.
    for ($i = 0; $i < self::AWS_CLOUD_SUBNET_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      // Update tags.
      $this->updateTagsInMockData($num - 1, 'Subnets', 'Name', '', FALSE);
    }

    // Make sure listing.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/subnet");
    for ($i = 0; $i < self::AWS_CLOUD_SUBNET_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkExists($edit[$i]['name']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_SUBNET_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['subnet_id']);
    }

    // Delete name tags.
    for ($i = 0; $i < self::AWS_CLOUD_SUBNET_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      // Update tags.
      $this->updateTagsInMockData($num - 1, 'Subnets', 'Name', '', TRUE);
    }

    // Make sure listing.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/subnet");
    for ($i = 0; $i < self::AWS_CLOUD_SUBNET_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['subnet_id']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_SUBNET_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['subnet_id']);
    }

    // Update tags.
    for ($i = 0; $i < self::AWS_CLOUD_SUBNET_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      $this->updateTagsInMockData($num - 1, 'Subnets', 'Name', $edit[$i]['name'], FALSE);
    }

    // Make sure listing.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/subnet");
    for ($i = 0; $i < self::AWS_CLOUD_SUBNET_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkNotExists($edit[$i]['name']);
      $this->assertSession()->linkExists($add[$i]['subnet_id']);
    }

    // Click 'Refresh'.
    $this->clickLink($this->t('Refresh'));
    // Make sure listing.
    for ($i = 0; $i < self::AWS_CLOUD_SUBNET_REPEAT_COUNT; $i++) {
      $this->assertSession()->linkExists($edit[$i]['name']);
    }

    // Delete Subnet.
    for ($i = 0; $i < self::AWS_CLOUD_SUBNET_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deleteSubnetMockData($i);

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/subnet/$num/delete");
      $this->assertSession()->statusCodeEquals(200);

      $this->drupalPostForm(
        "/clouds/aws_cloud/$cloud_context/subnet/$num/delete",
        [],
        $this->t('Delete')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'The AWS Cloud Subnet @name has been deleted.',
        ['@name' => $edit[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/subnet");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($edit[$i]['name']);
    }
  }

  /**
   * Tests deleting subnets with bulk operation.
   */
  public function testSubnetBulk() {
    $cloud_context = $this->cloudContext;
    for ($i = 0; $i < self::AWS_CLOUD_SUBNET_REPEAT_COUNT; $i++) {
      // Create Subnets.
      $subnets = $this->createSubnetsRandomTestFormData();
      $index = 0;
      $entities = [];
      foreach ($subnets ?: [] as $subnet) {
        $entities[] = $this->createSubnetTestEntity($index++, $subnet['SubnetId'], $subnet['Name'], $cloud_context);
      }
      $this->doTestEntityBulk('subnet', $entities);
    }
  }

}
