<?php

namespace Drupal\k8s\Plugin\cloud\server_template;

use Drupal\cloud\Entity\CloudServerTemplateInterface;
use Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface;
use Drupal\cloud\Plugin\cloud\server_template\CloudServerTemplatePluginInterface;
use Drupal\cloud\Service\EntityLinkRendererInterface;
use Drupal\Component\Plugin\PluginBase;
use Drupal\Component\Serialization\Yaml;
use Drupal\Component\Utility\Html;
use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\File\Exception\NotRegularDirectoryException;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\k8s\Entity\K8sEntityBase;
use Drupal\k8s\Service\K8sServiceException;
use Drupal\k8s\Service\K8sServiceInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * K8s Cloud server template plugin.
 */
class K8sCloudServerTemplatePlugin extends PluginBase implements CloudServerTemplatePluginInterface, ContainerFactoryPluginInterface {

  use StringTranslationTrait;

  /**
   * The K8s Service.
   *
   * @var \Drupal\k8s\Service\K8sServiceInterface
   */
  protected $k8sService;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\Messenger
   */
  protected $messenger;

  /**
   * The Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The UUID service.
   *
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  protected $uuidService;

  /**
   * Current login user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The cloud service provider plugin manager (CloudConfigPluginManager).
   *
   * @var \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface
   */
  protected $cloudConfigPluginManager;

  /**
   * Entity link renderer object.
   *
   * @var \Drupal\cloud\Service\EntityLinkRendererInterface
   */
  protected $entityLinkRenderer;

  /**
   * File system object.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * K8sCloudServerTemplatePlugin constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\k8s\Service\K8sServiceInterface $k8s_service
   *   The Amazon EC2 Service.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   The Messenger service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Component\Uuid\UuidInterface $uuid_service
   *   The uuid service.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   Current login user.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   * @param \Drupal\cloud\Plugin\cloud\config\CloudConfigPluginManagerInterface $cloud_config_plugin_manager
   *   The cloud service provider plugin manager (CloudConfigPluginManager).
   * @param \Drupal\cloud\Service\EntityLinkRendererInterface $entity_link_renderer
   *   The entity link render service.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   */
  public function __construct(
  array $configuration,
    $plugin_id,
    $plugin_definition,
    K8sServiceInterface $k8s_service,
    Messenger $messenger,
    EntityTypeManagerInterface $entity_type_manager,
    UuidInterface $uuid_service,
    AccountProxyInterface $current_user,
    ConfigFactoryInterface $config_factory,
    TranslationInterface $string_translation,
    CloudConfigPluginManagerInterface $cloud_config_plugin_manager,
    EntityLinkRendererInterface $entity_link_renderer,
    FileSystemInterface $file_system
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->k8sService = $k8s_service;
    $this->messenger = $messenger;
    $this->entityTypeManager = $entity_type_manager;
    $this->uuidService = $uuid_service;
    $this->currentUser = $current_user;
    $this->configFactory = $config_factory;
    $this->stringTranslation = $string_translation;
    $this->cloudConfigPluginManager = $cloud_config_plugin_manager;
    $this->entityLinkRenderer = $entity_link_renderer;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('k8s'),
      $container->get('messenger'),
      $container->get('entity_type.manager'),
      $container->get('uuid'),
      $container->get('current_user'),
      $container->get('config.factory'),
      $container->get('string_translation'),
      $container->get('plugin.manager.cloud_config_plugin'),
      $container->get('entity.link_renderer'),
      $container->get('file_system')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityBundleName() {
    return $this->pluginDefinition['entity_bundle'];
  }

  /**
   * {@inheritdoc}
   */
  public function launch(CloudServerTemplateInterface $cloud_server_template, FormStateInterface $form_state = NULL) {
    $this->k8sService->setCloudContext($cloud_server_template->getCloudContext());
    $k8s_obj = $cloud_server_template->get('field_object')->value;
    $source_type = $cloud_server_template->get('field_source_type')->value;

    $route = [
      'route_name' => 'entity.cloud_server_template.canonical',
      'params' => [
        'cloud_server_template' => $cloud_server_template->id(),
        'cloud_context' => $cloud_server_template->getCloudContext(),
      ],
    ];

    if ($source_type === 'git') {
      $this->launchByGitYamlFiles($cloud_server_template);
      return $route;
    }

    switch ($k8s_obj) {
      case 'pod':
        $route = $this->launchPod($cloud_server_template);
        break;

      case 'deployment':
        $route = $this->launchDeployment($cloud_server_template);
        break;

      case 'cluster_role':
        $route = $this->launchClusterRole($cloud_server_template);
        break;

      case 'cluster_role_binding':
        $this->launchClusterRoleBinding($cloud_server_template);
        break;

      case 'role_binding':
        $this->launchRoleBinding($cloud_server_template);
        break;

      case 'api_service':
        $this->launchApiService($cloud_server_template);
        break;

      case 'service_account':
        $this->launchServiceAccount($cloud_server_template);
        break;

      case 'service':
        $route = $this->launchService($cloud_server_template);
        break;

      case 'cron_job':
        $route = $this->launchCronJob($cloud_server_template);
        break;

      default:
        $this->messenger->addError($this->t('@object launch not supported.', ['@object' => $k8s_obj]));

    }

    return $route;
  }

  /**
   * {@inheritdoc}
   */
  public function buildListHeader() {
    return [
      [
        'data' => $this->t('Namespace'),
        'specifier' => 'field_namespace',
        'field' => 'field_namespace',
      ],
      [
        'data' => $this->t('Object'),
        'specifier' => 'field_object',
        'field' => 'field_object',
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildListRow(CloudServerTemplateInterface $entity) {
    $row['field_namespace'] = [
      'data' => $this->entityLinkRenderer->renderViewElement(
        $entity->get('field_namespace')->value,
        'k8s_namespace',
        'name',
        []
      ),
    ];
    $row['field_object'] = [
      'data' => $this->renderField($entity, 'field_object'),
    ];
    return $row;
  }

  /**
   * Render a server template entity field.
   *
   * @param \Drupal\cloud\Entity\CloudServerTemplateInterface $entity
   *   The server template entity.
   * @param string $field_name
   *   The field to render.
   * @param string $view
   *   The view to render.
   *
   * @return array
   *   A fully loaded render array for that field or empty array.
   */
  private function renderField(CloudServerTemplateInterface $entity, $field_name, $view = 'default') {
    $field = [];
    if ($entity->hasField($field_name)) {
      $field = $entity->get($field_name)->view($view);
      // Hide the label.
      $field['#label_display'] = 'hidden';
    }
    return $field;
  }

  /**
   * Launch a K8s deployment from a server template.
   *
   * @param \Drupal\cloud\Entity\CloudServerTemplateInterface $cloud_server_template
   *   Cloud server template interface.
   *
   * @return array
   *   The route to redirect after launch.
   *
   * @throws \Drupal\k8s\Service\K8sServiceException
   *   Thrown when unable to launch deployment.
   */
  private function launchDeployment(CloudServerTemplateInterface $cloud_server_template) {
    $params = Yaml::decode($cloud_server_template->get('field_detail')->value);
    $params['metadata']['annotations'][K8sEntityBase::ANNOTATION_LAUNCHED_APPLICATION_ID] = $cloud_server_template->id();
    $route = [
      'route_name' => 'view.k8s_deployment.list',
      'params' => ['cloud_context' => $cloud_server_template->getCloudContext()],
    ];
    try {
      $result = $this->k8sService->createDeployment(
        $cloud_server_template->get('field_namespace')->value,
        $params
      );

      $this->k8sService->updateDeployments([
        'metadata.name' => $result['metadata']['name'],
      ], FALSE);

      // Update creation_yaml field of k8s_pod entity.
      $entities = $this->entityTypeManager->getStorage('k8s_deployment')->loadByProperties(
        [
          'cloud_context' => $cloud_server_template->getCloudContext(),
          'name' => $result['metadata']['name'],
        ]
      );
      if (!empty($entities)) {
        $entity = reset($entities);
        $entity->setCreationYaml($cloud_server_template->get('field_detail')->value);
        $entity->save();

        $this->messenger->addStatus($this->t('Deployment <a href=":url">%name</a> launched.', [
          '%name' => $result['metadata']['name'],
          ':url' => $entity->toUrl('canonical')->toString(),
        ]));
        $cloud_server_template->get('field_launch_resources')->appendItem(['item_key' => $entity->getEntityTypeId(), 'item_value' => $entity->id()]);

        // Identify whether called by 'launchByGitYamlFiles'.
        // Parameters are 2 when called by the function.
        // And the cloud server template shouldn't be saved.
        if (count(func_get_args()) === 1) {
          $cloud_server_template->validate();
          $cloud_server_template->save();
        }
      }
      else {
        $this->messenger->addStatus($this->t('Deployment %name launched.', ['%name' => $result['metadata']['name']]));
      }

    }
    catch (K8sServiceException $e) {
      $message = $this->t('The @label "%label" could not be launched.', [
        '@label' => $cloud_server_template->getEntityType()->getLabel(),
        '%label' => $cloud_server_template->label(),
      ]);
      $this->messenger->addError($message);
      $route = [
        'route_name' => 'entity.cloud_server_template.canonical',
        'params' => [
          'cloud_server_template' => $cloud_server_template->id(),
          'cloud_context' => $cloud_server_template->getCloudContext(),
        ],
      ];
    }
    return $route;
  }

  /**
   * Launch a K8s pod from a server template.
   *
   * @param \Drupal\cloud\Entity\CloudServerTemplateInterface $cloud_server_template
   *   Cloud server template interface.
   *
   * @return array
   *   The route to redirect after launch.
   *
   * @throws \Drupal\k8s\Service\K8sServiceException
   *   Thrown when unable to launch pod.
   */
  private function launchPod(CloudServerTemplateInterface $cloud_server_template) {
    $params = Yaml::decode($cloud_server_template->get('field_detail')->value);
    $params['metadata']['annotations'][K8sEntityBase::ANNOTATION_LAUNCHED_APPLICATION_ID] = $cloud_server_template->id();
    $route = [
      'route_name' => 'view.k8s_pod.list',
      'params' => ['cloud_context' => $cloud_server_template->getCloudContext()],
    ];
    try {
      $result = $this->k8sService->createPod(
        $cloud_server_template->get('field_namespace')->value,
        $params
      );

      $this->k8sService->updatePods([
        'metadata.name' => $result['metadata']['name'],
      ], FALSE);

      // Update creation_yaml field of k8s_pod entity.
      $entities = $this->entityTypeManager->getStorage('k8s_pod')->loadByProperties(
        [
          'cloud_context' => $cloud_server_template->getCloudContext(),
          'name' => $result['metadata']['name'],
        ]
      );
      if (!empty($entities)) {
        $entity = reset($entities);
        $entity->setCreationYaml($cloud_server_template->get('field_detail')->value);
        $entity->save();

        $this->messenger->addStatus($this->t('Pod <a href=":url">%name</a> launched.', [
          '%name' => $result['metadata']['name'],
          ':url' => $entity->toUrl('canonical')->toString(),
        ]));
        $cloud_server_template->get('field_launch_resources')->appendItem(['item_key' => $entity->getEntityTypeId(), 'item_value' => $entity->id()]);

        // Identify whether called by 'launchByGitYamlFiles'.
        // Parameters are 2 when called by the function.
        // And the cloud server template shouldn't be saved.
        if (count(func_get_args()) === 1) {
          $cloud_server_template->validate();
          $cloud_server_template->save();
        }
      }
      else {
        $this->messenger->addStatus($this->t('Pod %name launched.', ['%name' => $result['metadata']['name']]));
      }
    }
    catch (K8sServiceException $e) {
      $message = $this->t('The @label "%label" could not be launched.', [
        '@label' => $cloud_server_template->getEntityType()->getLabel(),
        '%label' => $cloud_server_template->label(),
      ]);
      $this->messenger->addError($message);
      $route = [
        'route_name' => 'entity.cloud_server_template.canonical',
        'params' => [
          'cloud_server_template' => $cloud_server_template->id(),
          'cloud_context' => $cloud_server_template->getCloudContext(),
        ],
      ];
    }
    return $route;
  }

  /**
   * Launch a K8s cluster role from a server template.
   *
   * @param \Drupal\cloud\Entity\CloudServerTemplateInterface $cloud_server_template
   *   Cloud server template interface.
   *
   * @return array
   *   The route to redirect after launch.
   *
   * @throws \Drupal\k8s\Service\K8sServiceException
   *   Thrown when unable to launch cluster role.
   */
  private function launchClusterRole(CloudServerTemplateInterface $cloud_server_template) {
    $params = Yaml::decode($cloud_server_template->get('field_detail')->value);
    $params['metadata']['annotations'][K8sEntityBase::ANNOTATION_LAUNCHED_APPLICATION_ID] = $cloud_server_template->id();
    $route = [
      'route_name' => 'view.k8s_cluster_role.list',
      'params' => ['cloud_context' => $cloud_server_template->getCloudContext()],
    ];
    try {
      $result = $this->k8sService->createClusterRole(
        $params
      );

      $this->k8sService->updateClusterRoles([
        'metadata.name' => $result['metadata']['name'],
      ], FALSE);

      // Update creation_yaml field of k8s_pod entity.
      $entities = $this->entityTypeManager->getStorage('k8s_cluster_role')->loadByProperties(
        [
          'cloud_context' => $cloud_server_template->getCloudContext(),
          'name' => $result['metadata']['name'],
        ]
      );
      if (!empty($entities)) {
        $entity = reset($entities);
        $entity->setCreationYaml($cloud_server_template->get('field_detail')->value);
        $entity->save();

        $this->messenger->addStatus($this->t('ClusterRole <a href=":url">%name</a> launched.', [
          '%name' => $result['metadata']['name'],
          ':url' => $entity->toUrl('canonical')->toString(),
        ]));
        $cloud_server_template->get('field_launch_resources')->appendItem(['item_key' => $entity->getEntityTypeId(), 'item_value' => $entity->id()]);

        // Identify whether called by 'launchByGitYamlFiles'.
        // Parameters are 2 when called by the function.
        // And the cloud server template shouldn't be saved.
        if (count(func_get_args()) === 1) {
          $cloud_server_template->validate();
          $cloud_server_template->save();
        }
      }
      else {
        $this->messenger->addStatus($this->t('ClusterRole %name launched.', ['%name' => $result['metadata']['name']]));
      }
    }
    catch (K8sServiceException $e) {
      $message = $this->t('The @label "%label" could not be launched.', [
        '@label' => $cloud_server_template->getEntityType()->getLabel(),
        '%label' => $cloud_server_template->label(),
      ]);
      $this->messenger->addError($message);
      $route = [
        'route_name' => 'entity.cloud_server_template.canonical',
        'params' => [
          'cloud_server_template' => $cloud_server_template->id(),
          'cloud_context' => $cloud_server_template->getCloudContext(),
        ],
      ];
    }
    return $route;
  }

  /**
   * Launch a K8s cluster role binding from a server template.
   *
   * @param \Drupal\cloud\Entity\CloudServerTemplateInterface $cloud_server_template
   *   Cloud server template interface.
   *
   * @throws \Drupal\k8s\Service\K8sServiceException
   *   Thrown when unable to launch cluster role binding.
   */
  private function launchClusterRoleBinding(CloudServerTemplateInterface $cloud_server_template) {
    $params = Yaml::decode($cloud_server_template->get('field_detail')->value);
    $params['metadata']['annotations'][K8sEntityBase::ANNOTATION_LAUNCHED_APPLICATION_ID] = $cloud_server_template->id();

    try {
      $result = $this->k8sService->createClusterRoleBinding(
        $params
      );

      $this->k8sService->updateClusterRoleBindings([
        'metadata.name' => $result['metadata']['name'],
      ], FALSE);

      // Get created entity.
      $entities = $this->entityTypeManager->getStorage('k8s_cluster_role_binding')->loadByProperties(
        [
          'cloud_context' => $cloud_server_template->getCloudContext(),
          'name' => $result['metadata']['name'],
        ]
      );

      if (!empty($entities)) {
        $entity = reset($entities);

        $this->messenger->addStatus($this->t('ClusterRoleBinding <a href=":url">%name</a> launched.', [
          '%name' => $result['metadata']['name'],
          ':url' => $entity->toUrl('canonical')->toString(),
        ]));
        $cloud_server_template->get('field_launch_resources')->appendItem(['item_key' => $entity->getEntityTypeId(), 'item_value' => $entity->id()]);

        // Identify whether called by 'launchByGitYamlFiles'.
        // Parameters are 2 when called by the function.
        // And the cloud server template shouldn't be saved.
        if (count(func_get_args()) === 1) {
          $cloud_server_template->validate();
          $cloud_server_template->save();
        }
      }
      else {
        $this->messenger->addStatus($this->t('ClusterRoleBinding %name launched.', ['%name' => $result['metadata']['name']]));
      }

    }
    catch (K8sServiceException $e) {
      $message = $this->t('The @label "%label" could not be launched.', [
        '@label' => $cloud_server_template->getEntityType()->getLabel(),
        '%label' => $cloud_server_template->label(),
      ]);
      $this->messenger->addError($message);
    }
  }

  /**
   * Launch a K8s role binding from a server template.
   *
   * @param \Drupal\cloud\Entity\CloudServerTemplateInterface $cloud_server_template
   *   Cloud server template interface.
   *
   * @throws \Drupal\k8s\Service\K8sServiceException
   *   Thrown when unable to launch role binding.
   */
  private function launchRoleBinding(CloudServerTemplateInterface $cloud_server_template) {
    $params = Yaml::decode($cloud_server_template->get('field_detail')->value);
    $params['metadata']['annotations'][K8sEntityBase::ANNOTATION_LAUNCHED_APPLICATION_ID] = $cloud_server_template->id();

    try {
      $result = $this->k8sService->createRoleBinding(
       $cloud_server_template->get('field_namespace')->value,
        $params
      );

      $this->k8sService->updateRoleBindings([
        'metadata.name' => $result['metadata']['name'],
      ], FALSE);

      // Get created entity.
      $entities = $this->entityTypeManager->getStorage('k8s_role_binding')->loadByProperties(
        [
          'cloud_context' => $cloud_server_template->getCloudContext(),
          'name' => $result['metadata']['name'],
        ]
      );
      if (!empty($entities)) {
        $entity = reset($entities);

        $this->messenger->addStatus($this->t('RoleBinding <a href=":url">%name</a> launched.', [
          '%name' => $result['metadata']['name'],
          ':url' => $entity->toUrl('canonical')->toString(),
        ]));
        $cloud_server_template->get('field_launch_resources')->appendItem(['item_key' => $entity->getEntityTypeId(), 'item_value' => $entity->id()]);

        // Identify whether called by 'launchByGitYamlFiles'.
        // Parameters are 2 when called by the function.
        // And the cloud server template shouldn't be saved.
        if (count(func_get_args()) === 1) {
          $cloud_server_template->validate();
          $cloud_server_template->save();
        }
      }
      else {
        $this->messenger->addStatus($this->t('RoleBinding %name launched.', ['%name' => $result['metadata']['name']]));
      }

    }
    catch (K8sServiceException $e) {
      $message = $this->t('The @label "%label" could not be launched.', [
        '@label' => $cloud_server_template->getEntityType()->getLabel(),
        '%label' => $cloud_server_template->label(),
      ]);
      $this->messenger->addError($message);
    }
  }

  /**
   * Launch a K8s api service from a server template.
   *
   * @param \Drupal\cloud\Entity\CloudServerTemplateInterface $cloud_server_template
   *   Cloud server template interface.
   *
   * @throws \Drupal\k8s\Service\K8sServiceException
   *   Thrown when unable to launch Api service.
   */
  private function launchApiService(CloudServerTemplateInterface $cloud_server_template) {
    $params = Yaml::decode($cloud_server_template->get('field_detail')->value);
    $params['metadata']['annotations'][K8sEntityBase::ANNOTATION_LAUNCHED_APPLICATION_ID] = $cloud_server_template->id();

    try {
      $result = $this->k8sService->createAPIService(
        $params
      );

      $this->k8sService->updateAPIServices([
        'metadata.name' => $result['metadata']['name'],
      ], FALSE);

      // Get created entity.
      $entities = $this->entityTypeManager->getStorage('k8s_api_service')->loadByProperties(
        [
          'cloud_context' => $cloud_server_template->getCloudContext(),
          'name' => $result['metadata']['name'],
        ]
      );

      if (!empty($entities)) {
        $entity = reset($entities);
        $this->messenger->addStatus($this->t('APIService <a href=":url">%name</a> launched.', [
          '%name' => $result['metadata']['name'],
          ':url' => $entity->toUrl('canonical')->toString(),
        ]));
        $cloud_server_template->get('field_launch_resources')->appendItem(['item_key' => $entity->getEntityTypeId(), 'item_value' => $entity->id()]);

        // Identify whether called by 'launchByGitYamlFiles'.
        // Parameters are 2 when called by the function.
        // And the cloud server template shouldn't be saved.
        if (count(func_get_args()) === 1) {
          $cloud_server_template->validate();
          $cloud_server_template->save();
        }
      }
      else {
        $this->messenger->addStatus($this->t('APIService %name launched.', ['%name' => $result['metadata']['name']]));
      }

    }
    catch (K8sServiceException $e) {
      $message = $this->t('The @label "%label" could not be launched.', [
        '@label' => $cloud_server_template->getEntityType()->getLabel(),
        '%label' => $cloud_server_template->label(),
      ]);
      $this->messenger->addError($message);
    }
  }

  /**
   * Launch a K8s service account from a server template.
   *
   * @param \Drupal\cloud\Entity\CloudServerTemplateInterface $cloud_server_template
   *   Cloud server template interface.
   *
   * @throws \Drupal\k8s\Service\K8sServiceException
   *   Thrown when unable to launch service account.
   */
  private function launchServiceAccount(CloudServerTemplateInterface $cloud_server_template) {
    $params = Yaml::decode($cloud_server_template->get('field_detail')->value);
    $params['metadata']['annotations'][K8sEntityBase::ANNOTATION_LAUNCHED_APPLICATION_ID] = $cloud_server_template->id();

    try {
      $result = $this->k8sService->createServiceAccount(
        $cloud_server_template->get('field_namespace')->value,
        $params
      );

      $this->k8sService->updateServiceAccounts([
        'metadata.name' => $result['metadata']['name'],
      ], FALSE);

      // Get created entity.
      $entities = $this->entityTypeManager->getStorage('k8s_service_account')->loadByProperties(
        [
          'cloud_context' => $cloud_server_template->getCloudContext(),
          'name' => $result['metadata']['name'],
        ]
      );
      if (!empty($entities)) {
        $entity = reset($entities);
        $this->messenger->addStatus($this->t('ServiceAccount <a href=":url">%name</a> launched.', [
          '%name' => $result['metadata']['name'],
          ':url' => $entity->toUrl('canonical')->toString(),
        ]));
        $cloud_server_template->get('field_launch_resources')->appendItem(['item_key' => $entity->getEntityTypeId(), 'item_value' => $entity->id()]);

        // Identify whether called by 'launchByGitYamlFiles'.
        // Parameters are 2 when called by the function.
        // And the cloud server template shouldn't be saved.
        if (count(func_get_args()) === 1) {
          $cloud_server_template->validate();
          $cloud_server_template->save();
        }
      }
      else {
        $this->messenger->addStatus($this->t('ServiceAccount %name launched.', ['%name' => $result['metadata']['name']]));
      }

    }
    catch (K8sServiceException $e) {
      $message = $this->t('The @label "%label" could not be launched.', [
        '@label' => $cloud_server_template->getEntityType()->getLabel(),
        '%label' => $cloud_server_template->label(),
      ]);
      $this->messenger->addError($message);
    }
  }

  /**
   * Launch a K8s service from a server template.
   *
   * @param \Drupal\cloud\Entity\CloudServerTemplateInterface $cloud_server_template
   *   Cloud server template interface.
   *
   * @return array
   *   The route to redirect after launch.
   *
   * @throws \Drupal\k8s\Service\K8sServiceException
   *   Thrown when unable to launch service.
   */
  private function launchService(CloudServerTemplateInterface $cloud_server_template) {
    $params = Yaml::decode($cloud_server_template->get('field_detail')->value);
    $params['metadata']['annotations'][K8sEntityBase::ANNOTATION_LAUNCHED_APPLICATION_ID] = $cloud_server_template->id();
    $route = [
      'route_name' => 'view.k8s_service.list',
      'params' => ['cloud_context' => $cloud_server_template->getCloudContext()],
    ];
    try {
      $result = $this->k8sService->createService(
        $cloud_server_template->get('field_namespace')->value,
        $params
      );

      $this->k8sService->updateServices([
        'metadata.name' => $result['metadata']['name'],
      ], FALSE);

      // Update creation_yaml field of k8s_pod entity.
      $entities = $this->entityTypeManager->getStorage('k8s_service')->loadByProperties(
        [
          'cloud_context' => $cloud_server_template->getCloudContext(),
          'name' => $result['metadata']['name'],
        ]
      );
      if (!empty($entities)) {
        $entity = reset($entities);
        $entity->setCreationYaml($cloud_server_template->get('field_detail')->value);
        $entity->save();

        $this->messenger->addStatus($this->t('Service <a href=":url">%name</a> launched.', [
          '%name' => $result['metadata']['name'],
          ':url' => $entity->toUrl('canonical')->toString(),
        ]));
        $cloud_server_template->get('field_launch_resources')->appendItem(['item_key' => $entity->getEntityTypeId(), 'item_value' => $entity->id()]);

        // Identify whether called by 'launchByGitYamlFiles'.
        // Parameters are 2 when called by the function.
        // And the cloud server template shouldn't be saved.
        if (count(func_get_args()) === 1) {
          $cloud_server_template->validate();
          $cloud_server_template->save();
        }
      }
      else {
        $this->messenger->addStatus($this->t('Service %name launched.', ['%name' => $result['metadata']['name']]));
      }

    }
    catch (K8sServiceException $e) {
      $message = $this->t('The @label "%label" could not be launched.', [
        '@label' => $cloud_server_template->getEntityType()->getLabel(),
        '%label' => $cloud_server_template->label(),
      ]);
      $this->messenger->addError($message);
      $route = [
        'route_name' => 'entity.cloud_server_template.canonical',
        'params' => [
          'cloud_server_template' => $cloud_server_template->id(),
          'cloud_context' => $cloud_server_template->getCloudContext(),
        ],
      ];
    }
    return $route;
  }

  /**
   * Launch a K8s cron job from a server template.
   *
   * @param \Drupal\cloud\Entity\CloudServerTemplateInterface $cloud_server_template
   *   Cloud server template interface.
   *
   * @return array
   *   The route to redirect after launch.
   */
  private function launchCronJob(CloudServerTemplateInterface $cloud_server_template) {
    $params = Yaml::decode($cloud_server_template->get('field_detail')->value);
    $params['metadata']['annotations'][K8sEntityBase::ANNOTATION_LAUNCHED_APPLICATION_ID] = $cloud_server_template->id();
    $route = [
      'route_name' => 'view.k8s_cron_job.list',
      'params' => ['cloud_context' => $cloud_server_template->getCloudContext()],
    ];
    try {
      $result = $this->k8sService->createCronJob(
        $cloud_server_template->get('field_namespace')->value,
        $params
      );

      $this->k8sService->updateCronJobs([
        'metadata.name' => $result['metadata']['name'],
      ], FALSE);

      // Get created entity.
      $entities = $this->entityTypeManager->getStorage('k8s_cron_job')->loadByProperties(
        [
          'cloud_context' => $cloud_server_template->getCloudContext(),
          'name' => $result['metadata']['name'],
        ]
      );

      if (!empty($entities)) {
        $entity = reset($entities);

        $this->messenger->addStatus($this->t('CronJob <a href=":url">%name</a> launched.', [
          '%name' => $result['metadata']['name'],
          ':url' => $entity->toUrl('canonical')->toString(),
        ]));
        $cloud_server_template->get('field_launch_resources')->appendItem(['item_key' => $entity->getEntityTypeId(), 'item_value' => $entity->id()]);

        // Identify whether called by 'launchByGitYamlFiles'.
        // Parameters are 2 when called by the function.
        // And the cloud server template shouldn't be saved.
        if (count(func_get_args()) === 1) {
          $cloud_server_template->validate();
          $cloud_server_template->save();
        }
      }
      else {
        $this->messenger->addStatus($this->t('CronJob %name launched.', ['%name' => $result['metadata']['name']]));
      }

    }
    catch (K8sServiceException $e) {
      $message = $this->t('The @label "%label" could not be launched.', [
        '@label' => $cloud_server_template->getEntityType()->getLabel(),
        '%label' => $cloud_server_template->label(),
      ]);
      $this->messenger->addError($message);
      $route = [
        'route_name' => 'entity.cloud_server_template.canonical',
        'params' => [
          'cloud_server_template' => $cloud_server_template->id(),
          'cloud_context' => $cloud_server_template->getCloudContext(),
        ],
      ];
    }
    return $route;
  }

  /**
   * Launch by YAML files in git repository.
   *
   * @param \Drupal\cloud\Entity\CloudServerTemplateInterface $cloud_server_template
   *   Cloud server template interface.
   */
  private function launchByGitYamlFiles(CloudServerTemplateInterface $cloud_server_template) {
    $git_url_value = $cloud_server_template->get('field_yaml_url')->getValue();
    $git_url = $git_url_value[0]['uri'];
    $git_paths = $cloud_server_template->get('field_git_path')->getValue();
    $git_username = $cloud_server_template->get('field_git_username')->value;
    $git_password = $cloud_server_template->get('field_git_password')->value;

    $tmp_dir = $this->fileSystem->getTempDirectory();
    $name = Html::getId($cloud_server_template->getName() . ' ' . $cloud_server_template->id());
    $dir_name = $this->fileSystem->getDestinationFilename($tmp_dir . "/$name", FileSystemInterface::EXISTS_RENAME);

    // Make the temporary working directory.
    $result = $this->fileSystem->mkdir($dir_name);

    if (!$result) {
      $this->messenger->addError($this->t("Unable to create the working directory."));
      return;
    }
    if ($git_username) {
      $account = rawurlencode($git_username);
      if ($git_password) {
        $account .= ":" . rawurlencode($git_password);
      }
      $git_url = str_replace("://", "://$account@", $git_url);
    }
    $output = [];
    $return_var = 0;
    exec("git clone $git_url $dir_name", $output, $return_var);
    if ($return_var !== 0) {
      $this->messenger->addError($this->t("Unable to clone the git repository."));
      return;
    };

    $match = '/.*/';

    $files = [];
    try {
      foreach ($git_paths ?: [] as $path) {
        $files = array_merge($files, $this->fileSystem->scanDirectory($dir_name . $path['value'], $match));
      }
    }
    catch (NotRegularDirectoryException $e) {
      $this->messenger->addError($this->t("Git Resource Path might not be correct."));
      $this->fileSystem->deleteRecursive($dir_name);
      return;
    }

    if (!empty($files)) {
      $objects = [];
      foreach ($files ?: [] as $file) {
        $file_contents = file_get_contents($file->uri);
        if (!$file_contents) {
          $this->messenger->addError($this->t("Unable to get contents from the file [%file].", ["%file" => $file->filename]));
        }
        else {
          try {
            $yaml = Yaml::decode($file_contents);
            if (!is_array($yaml)) {
              $this->messenger->addError($this->t("The file [%file] isn't YAML format.", ['%file' => $file->filename]));
            }
            elseif (!isset($yaml['kind'])) {
              $this->messenger->addError($this->t("No 'Kind' element found in the file [%file].", ['%file' => $file->filename]));
            }
            else {
              $kind = $yaml['kind'];
              $templates = k8s_supported_cloud_server_templates();
              $object = array_search($kind, $templates);
              if ($object === FALSE) {
                $this->messenger->addError($this->t("Unsupported 'Kind' element in the file [%file].", ['%file' => $file->filename]));
              }
              else {
                $cloud_server_template->get('field_detail')->setValue($file_contents);
                $function = "launch$kind";
                if (method_exists($this, $function)) {
                  $errors_before = count($this->messenger->messagesByType($this->messenger::TYPE_ERROR));
                  // Put an extra 2nd param to identify called by this function.
                  $this->$function($cloud_server_template, TRUE);
                  $errors_after = count($this->messenger->messagesByType($this->messenger::TYPE_ERROR));
                  if ($errors_before === $errors_after) {
                    $objects[] = $object;
                  }
                }
              }
            }
          }
          catch (\Exception $e) {
            $this->messenger->addError($this->t("Invalid Yaml format in the file [%file]:%message", [
              '%file' => $file->filename,
              '%message' => $e->getMessage(),
            ]));
          }
        }
      }

      $cloud_server_template->get('field_object')->setValue($objects);
      $cloud_server_template->get('field_detail')->setValue(NULL);
      $cloud_server_template->validate();
      $cloud_server_template->save();

    }
    $this->fileSystem->deleteRecursive($dir_name);
  }

}
