<?php

namespace Drupal\Tests\k8s\Functional;

/**
 * Tests K8s persistent volume claim.
 *
 * @group K8s
 */
class K8sPersistentVolumeClaimTest extends K8sTestCase {

  public const K8S_PERSISTENT_VOLUME_CLAIM_REPEAT_COUNT = 2;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    $namespaces = $this->createNamespacesRandomTestFormData();
    $this->createNamespaceTestEntity($namespaces[0]);
    $this->namespace = $namespaces[0]['name'];
    return [
      'list k8s persistent volume claim',
      'add k8s persistent volume claim',
      'edit k8s persistent volume claim',
      'delete k8s persistent volume claim',
      'view k8s namespace ' . $this->namespace,
    ];
  }

  /**
   * Tests CRUD for Persistent Volume Claim.
   */
  public function testPersistentVolumeClaim() {

    $cloud_context = $this->cloudContext;

    // List Persistent Volume Claim for K8s.
    $this->drupalGet("/clouds/k8s/$cloud_context/persistent_volume_claim");
    $this->assertSession()->statusCodeEquals(200);

    // Add a new Persistent Volume Claim.
    $add = $this->createPersistentVolumeClaimTestFormData(self::K8S_PERSISTENT_VOLUME_CLAIM_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_PERSISTENT_VOLUME_CLAIM_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $this->addPersistentVolumeClaimMockData($add[$i]);

      $num = $i + 1;
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/persistent_volume_claim/add",
        $add[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t('Kubernetes Persistent Volume Claim @name has been created.', ['@name' => $add[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/persistent_volume_claim");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($add[$i]['name']);
    }

    // Edit a Persistent Volume Claim.
    $edit = $this->createPersistentVolumeClaimTestFormData(self::K8S_PERSISTENT_VOLUME_CLAIM_REPEAT_COUNT, $this->namespace);
    for ($i = 0; $i < self::K8S_PERSISTENT_VOLUME_CLAIM_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->updatePersistentVolumeClaimMockData($edit[$i]);

      unset($edit[$i]['post_data']['namespace']);
      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/persistent_volume_claim/$num/edit",
        $edit[$i]['post_data'],
        $this->t('Save')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'Kubernetes Persistent Volume Claim @name has been updated.',
        ['@name' => $add[$i]['name']]
      ));
    }

    // Delete Persistent Volume Claim.
    for ($i = 0; $i < self::K8S_PERSISTENT_VOLUME_CLAIM_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $this->deletePersistentVolumeClaimMockData($add[$i]);

      $this->drupalPostForm(
        "/clouds/k8s/$cloud_context/persistent_volume_claim/$num/delete",
        [],
        $this->t('Delete')
      );
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextContains($this->t(
        'The Kubernetes Persistent Volume Claim @name has been deleted.',
        ['@name' => $add[$i]['name']]
      ));

      // Make sure listing.
      $this->drupalGet("/clouds/k8s/$cloud_context/persistent_volume_claim");
      $this->assertSession()->statusCodeEquals(200);
      $this->assertSession()->pageTextNotContains($add[$i]['name']);
    }
  }

  /**
   * Tests deleting persistent volume claim with bulk operation.
   */
  public function testPersistentVolumeClaimBulk() {

    for ($i = 0; $i < self::K8S_PERSISTENT_VOLUME_CLAIM_REPEAT_COUNT; $i++) {
      // Create Persistent Volume Claim.
      $persistent_volume_claims = $this->createPersistentVolumeClaimsRandomTestFormData();
      $entities = [];
      foreach ($persistent_volume_claims ?: [] as $persistent_volume_claim) {
        $entities[] = $this->createPersistentVolumeClaimTestEntity($persistent_volume_claim);
      }
      $this->deletePersistentVolumeClaimMockData($persistent_volume_claims[0]);
      $this->doTestEntityBulk('persistent_volume_claim', $entities);
    }
  }

}
