<?php

namespace Drupal\Tests\k8s\Traits;

use Drupal\Component\Utility\Random;

/**
 * The trait creating form data for k8s testing.
 */
trait K8sTestFormDataTrait {

  /**
   * Create random value using regex as '[a-z0-9]([-a-z0-9]*[a-z0-9])
   *
   * @param int $length
   *   The length of random value.
   *
   * @return string
   *   Random value.
   */
  private function makeRandomString($length) {
    $str = array_merge(range('a', 'z'), range('0', '9'));
    $r_str = NULL;
    for ($i = 0; $i < $length; $i++) {
      $r_str .= $str[rand(0, count($str) - 1)];
    }
    return $r_str;
  }

  /**
   * Create test data for cloud service provider (CloudConfig).
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   Test data.
   */
  protected function createCloudConfigTestFormData($repeat_count) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0, $num = 1; $i < $repeat_count; $i++, $num++) {

      $data[] = [
        'name[0][value]'              => sprintf('config-entity-%s - %s', $random->name(8, TRUE), date('Y/m/d H:i:s')),
        'cloud_context'               => strtolower($random->name(16, TRUE)),
        'field_api_server[0][value]'  => 'https://www.k8s-test.com/',
        'field_token[0][value]'       => $random->name(128, TRUE),
      ];
    }

    return $data;
  }

  /**
   * Create test data for k8s node.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   Test data.
   */
  protected function createNodeTestFormData($repeat_count) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $data[] = [
        'name' => sprintf('Node-%s - %s', $random->name(8, TRUE), date('Y/m/d H:i:s')),
      ];
    }

    return $data;
  }

  /**
   * Create test data for k8s namespace.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   Test data.
   */
  protected function createNamespaceTestFormData($repeat_count) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $data[] = [
        'name' => sprintf('Namespace-%s - %s', $random->name(8, TRUE), date('Y/m/d H:i:s')),
        'labels[0][item_key]' => 'key-' . $random->name(8, TRUE),
        'labels[0][item_value]' => 'value-' . $random->name(8, TRUE),
        'annotations[0][item_key]' => 'key-' . $random->name(8, TRUE),
        'annotations[0][item_value]' => 'value-' . $random->name(8, TRUE),
      ];
    }

    return $data;
  }

  /**
   * Create random namespaces.
   *
   * @return array
   *   Random namespaces array.
   */
  protected function createNamespacesRandomTestFormData() {
    $namespaces = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $key = 'key-' . $this->random->name(8, TRUE);
      $value = 'value-' . $this->random->name(16, TRUE);
      $namespaces[] = [
        'name' => sprintf('namespace-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
        'labels' => [
          [$key => $value],
        ],
        'annotations' => [
          [$key => $value],
        ],
      ];
    }

    return $namespaces;
  }

  /**
   * Create test data for k8s pod.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createPodTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: apps/v1
kind: Pod
metadata:
  name: $name
  labels:
    app: nginx
  annotations:
    key: value
spec:
  replicas: 3
  selector:
    matchLabels:
      app: nginx
  template:
    metadata:
      labels:
        app: nginx
    spec:
      containers:
      - name: nginx
        image: nginx:1.7.9
        ports:
        - containerPort: 80
EOS;
      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random pods.
   *
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Random pods array.
   */
  protected function createPodsRandomTestFormData($namespace) {
    $pods = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $pods[] = [
        'name' => sprintf('pod-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
        'namespace' => $namespace,
      ];
    }

    return $pods;
  }

  /**
   * Create test data for k8s deployment.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createDeploymentTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: v1
kind: Deployment
metadata:
  name: $name
spec:
  selector:
    app: MyApp
  ports:
    - protocol: TCP
      port: 80
      targetPort: 9376
EOS;
      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random deployments.
   *
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Random deployments array.
   */
  protected function createDeploymentsRandomTestFormData($namespace) {
    $deployments = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $deployments[] = [
        'name' => sprintf('deployment-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
        'namespace' => $namespace,
      ];
    }

    return $deployments;
  }

  /**
   * Create test data for k8s replica set.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createReplicaSetTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: v1
kind: ReplicaSet
metadata:
  name: $name
spec:
  selector:
    app: MyApp
  ports:
    - protocol: TCP
      port: 80
      targetPort: 9376
EOS;
      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random replica set.
   *
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Random replica sets array.
   */
  protected function createReplicaSetsRandomTestFormData($namespace) {
    $replica_sets = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $replica_sets[] = [
        'name' => sprintf('replica-set-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
        'namespace' => $namespace,
      ];
    }

    return $replica_sets;
  }

  /**
   * Create test data for k8s service.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createServiceTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: v1
kind: Service
metadata:
  name: $name
spec:
  selector:
    app: MyApp
  ports:
    - protocol: TCP
      port: 80
      targetPort: 9376
EOS;
      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random services.
   *
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Random services array.
   */
  protected function createServicesRandomTestFormData($namespace) {
    $services = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $services[] = [
        'name' => sprintf('service-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
        'namespace' => $namespace,
      ];
    }

    return $services;
  }

  /**
   * Create test data for k8s cron job.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createCronJobTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: batch/v1beta1
kind: CronJob
metadata:
  name: $name
spec:
  schedule: "* * */1 * *"
  jobTemplate:
    spec:
      template:
        spec:
          containers:
          - name: sleep
            image: alpine
            command: ["sh", "-c"]
            args:
            - |
              sleep 5
          restartPolicy: Never
EOS;
      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random cron jobs.
   *
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Random cron jobs array.
   */
  protected function createCronJobsRandomTestFormData($namespace) {
    $cron_jobs = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $cron_jobs[] = [
        'name' => sprintf('cron-job-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
        'namespace' => $namespace,
      ];
    }

    return $cron_jobs;
  }

  /**
   * Create test data for k8s job.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createJobTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: batch/v1
kind: Job
metadata:
  name: $name
spec:
  template:
    spec:
      containers:
      - name: sleep
        image: alpine
        command: ["sh",  "-c"]
        args:
        - |
          sleep 5
      restartPolicy: Never
  backoffLimit: 4
EOS;
      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random jobs.
   *
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Random jobs array.
   */
  protected function createJobsRandomTestFormData($namespace) {
    $jobs = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $jobs[] = [
        'name' => sprintf('job-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
        'namespace' => $namespace,
      ];
    }

    return $jobs;
  }

  /**
   * Create test data for k8s resource quota.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createResourceQuotaTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: v1
kind: ResourceQuota
metadata:
  name: $name
spec:
  hard:
    cpu: "10"
    memory: 20Gi
    pods: "10"
  scopeSelector:
    matchExpressions:
    - operator : In
      scopeName: PriorityClass
      values: ["medium"]
EOS;
      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random resource quotas.
   *
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Random resource quotas array.
   */
  protected function createResourceQuotasRandomTestFormData($namespace) {
    $resource_quotas = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $resource_quotas[] = [
        'name' => sprintf('resource-quota-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
        'namespace' => $namespace,
      ];
    }

    return $resource_quotas;
  }

  /**
   * Create test data for k8s limit range.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createLimitRangeTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: v1
kind: LimitRange
metadata:
  name: $name
spec:
  limits:
  - maxLimitRequestRatio:
      memory: 2
    type: Pod
EOS;
      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random limit ranges.
   *
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Random limit ranges array.
   */
  protected function createLimitRangesRandomTestFormData($namespace) {
    $limit_ranges = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $limit_ranges[] = [
        'name' => sprintf('limit-range-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
        'namespace' => $namespace,
      ];
    }

    return $limit_ranges;
  }

  /**
   * Create test data for k8s secret.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createSecretTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: v1
kind: Secret
metadata:
  name: $name
type: Opaque
data:
  username: YWRtaW4=
  password: MWYyZDFlMmU2N2Rm
EOS;
      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random secrets.
   *
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Random secrets array.
   */
  protected function createSecretsRandomTestFormData($namespace) {
    $secrets = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $secrets[] = [
        'name' => sprintf('secret-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
        'namespace' => $namespace,
      ];
    }

    return $secrets;
  }

  /**
   * Create test data for k8s config map.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createConfigMapTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: v1
kind: ConfigMap
metadata:
  name: $name
data:
  property1: hello
  property2: world
EOS;
      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random config maps.
   *
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Random config maps array.
   */
  protected function createConfigMapsRandomTestFormData($namespace) {
    $config_maps = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $config_maps[] = [
        'name' => sprintf('config-map-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
        'namespace' => $namespace,
      ];
    }

    return $config_maps;
  }

  /**
   * Create test data for k8s network policy.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createNetworkPolicyTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: networking.k8s.io/v1
kind: NetworkPolicy
metadata:
  name: $name
spec:
  policyTypes:
  - Ingress
  ingress:
  - from:
    - ipBlock:
        cidr: 172.17.0.0/16
        except:
        - 172.17.1.0/24
    ports:
    - protocol: TCP
      port: 6379
EOS;
      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random network policies.
   *
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Random network polices array.
   */
  protected function createNetworkPoliciesRandomTestFormData($namespace) {
    $network_polices = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $network_polices[] = [
        'name' => sprintf('network-policy-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
        'namespace' => $namespace,
      ];
    }

    return $network_polices;
  }

  /**
   * Create test data for k8s role.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createRoleTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: rbac.authorization.k8s.io/v1
kind: Role
metadata:
  name: $name
rules:
- apiGroups:
  - apps
  - extensions
  resources:
  - deployments
  verbs:
  - "get"
  - "list"
EOS;

      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random roles.
   *
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Random roles array.
   */
  protected function createRolesRandomTestFormData($namespace) {
    $roles = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $roles[] = [
        'name' => sprintf('role-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
        'namespace' => $namespace,
      ];
    }

    return $roles;
  }

  /**
   * Create test data for k8s cluster role.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   Test data.
   */
  protected function createClusterRoleTestFormData($repeat_count) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: rbac.authorization.k8s.io/v1
kind: ClusterRole
metadata:
  name: $name
rules:
- apiGroups:
  - apps
  - extensions
  resources:
  - deployments
  verbs:
  - "get"
  - "list"
EOS;

      $data[] = [
        'name' => $name,
        'post_data' => [
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random cluster roles.
   *
   * @return array
   *   Random cluster roles array.
   */
  protected function createClusterRolesRandomTestFormData() {
    $cluster_roles = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $cluster_roles[] = [
        'name' => sprintf('role-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
      ];
    }

    return $cluster_roles;
  }

  /**
   * Create test data for creating project.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   Test data.
   */
  protected function createProjectTestFormData($repeat_count) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $data[] = [
        'project_name' => sprintf('namespace-%s', $this->makeRandomString(8)),
        'username' => \Drupal::currentUser()->getAccountName(),
        "k8s_clusters[{$this->cloudContext}]" => $this->cloudContext,
        'k8s_enable_scheduling_time_based' => rand(0, 1),
        'k8s_startup_time_hour' => sprintf('%02d', rand(0, 23)),
        'k8s_startup_time_minute' => sprintf('%02d', rand(0, 59)),
        'k8s_stop_time_hour' => sprintf('%02d', rand(0, 23)),
        'k8s_stop_time_minute' => sprintf('%02d', rand(0, 59)),
        'k8s_enable_scheduling_resource_based' => rand(0, 1),
        'k8s_request_cpu' => rand(0, 1000),
        'k8s_request_memory' => rand(0, 1000),
        'k8s_pod_count' => rand(0, 20),
      ];
    }

    return $data;
  }

  /**
   * Create random cloud service provider (CloudConfig) data.
   *
   * @return array
   *   Random cloud service providers (CloudConfig).
   */
  protected function createCloudConfigRandomTestFormData() {
    $cloud_configs = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $cloud_configs[] = [
        'CloudContext' => "cloud_context-{$this->random->name(8)}",
        'Name' => sprintf('config-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(4, TRUE)),
      ];
    }

    return $cloud_configs;
  }

  /**
   * Create test data for k8s persistent volume.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   Test data.
   */
  protected function createPersistentVolumeTestFormData($repeat_count) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: v1
kind: PersistentVolume
metadata:
  name: $name
spec:
  capacity:
    storage: 5Gi
  volumeMode: Filesystem
  accessModes:
    - ReadWriteOnce
  persistentVolumeReclaimPolicy: Recycle
  storageClassName: slow
  mountOptions:
    - hard
    - nfsvers=4.1
  nfs:
    path: /tmp
    server: 172.17.0.2
EOS;
      $data[] = [
        'name' => $name,
        'post_data' => [
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random persistent volume.
   *
   * @return array
   *   Random persistent volume array.
   */
  protected function createPersistentVolumeRandomTestFormData() {
    $persistent_volume = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $persistent_volume[] = [
        'name' => sprintf('persistent-volume-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
      ];
    }

    return $persistent_volume;
  }

  /**
   * Create test data for k8s storage class.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   Test data.
   */
  protected function createStorageClassTestFormData($repeat_count) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: storage.k8s.io/v1
kind: StorageClass
metadata:
  name: $name
provisioner: kubernetes.io/aws-ebs
parameters:
  type: gp2
reclaimPolicy: Retain
allowVolumeExpansion: true
mountOptions:
  - debug
volumeBindingMode: Immediate
EOS;

      $data[] = [
        'name' => $name,
        'post_data' => [
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random storage classes.
   *
   * @return array
   *   Random storage classes array.
   */
  protected function createStorageClassesRandomTestFormData() {
    $storage_classes = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $storage_classes[] = [
        'name' => sprintf('storage-class-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
      ];
    }

    return $storage_classes;
  }

  /**
   * Create test data for k8s stateful set.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createStatefulSetTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: apps/v1
kind: StatefulSet
metadata:
  name: $name
spec:
  selector:
    app: MyApp
  ports:
    - protocol: TCP
      port: 80
      targetPort: 9376
EOS;

      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random stateful sets.
   *
   * @return array
   *   Random stateful sets array.
   */
  protected function createStatefulSetsRandomTestFormData() {
    $stateful_sets = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $stateful_sets[] = [
        'name' => sprintf('stateful-set-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
      ];
    }

    return $stateful_sets;
  }

  /**
   * Create test data for k8s ingress.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createIngressTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: extensions/v1beta1
kind: Ingress
metadata:
  name: $name
spec:
  rules:
    - host: sslexample.foo.com
      http:
        paths:
        - path: /
          backend:
            serviceName: service1
            servicePort: 80
  tls:
  - hosts:
    - sslexample.foo.com
    secretName: testsecret-tls1
EOS;

      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random ingresses.
   *
   * @return array
   *   Random ingresses array.
   */
  protected function createIngressesRandomTestFormData() {
    $ingresses = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $ingresses[] = [
        'name' => sprintf('ingress-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
      ];
    }

    return $ingresses;
  }

  /**
   * Create test data for k8s daemon set.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createDaemonSetTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: apps/v1
kind: DaemonSet
metadata:
  name: $name
  labels:
    app: $name
spec:
  selector:
    matchLabels:
      app: $name
  template:
    metadata:
      labels:
        app: $name
    spec:
      containers:
      - name: busybox
        image: busybox
        args:
        - sleep
        - "10000"
EOS;

      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random daemon sets.
   *
   * @return array
   *   Random daemon sets array.
   */
  protected function createDaemonSetsRandomTestFormData() {
    $daemon_sets = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $daemon_sets[] = [
        'name' => sprintf('daemon-set-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
      ];
    }

    return $daemon_sets;
  }

  /**
   * Create test data for k8s endpoint.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createEndpointTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: v1
kind: Endpoints
metadata:
  name: $name
subsets:
  ports:
    - protocol: TCP
      name: web
      port: 80
  addresses:
    ip: 192.168.233.92
    hostname: web-1
    nodeName: ip-192-168-243-46.us-west-2.compute.internal
EOS;

      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random endpoints.
   *
   * @return array
   *   Random endpoints array.
   */
  protected function createEndpointsRandomTestFormData() {
    $endpoints = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $endpoints[] = [
        'name' => sprintf('endpoint-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
      ];
    }

    return $endpoints;
  }

  /**
   * Create test data for k8s event.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   Test data.
   */
  protected function createEventTestFormData($repeat_count) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $data[] = [
        'name' => sprintf('Event-%s - %s', $random->name(8, TRUE), date('Y/m/d H:i:s')),
      ];
    }

    return $data;
  }

  /**
   * Create test data for k8s persistent volume claim.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createPersistentVolumeClaimTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: v1
kind: PersistentVolumeClaim
metadata:
 name: $name
spec:
 storageClassName: manual
 accessModes:
   - ReadWriteOnce
 resources:
   requests:
     storage: 1Gi
EOS;
      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random persistent volume claims.
   *
   * @return array
   *   Random persistent volume claims array.
   */
  protected function createPersistentVolumeClaimsRandomTestFormData() {
    $persistent_volume_claims = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $persistent_volume_claims[] = [
        'name' => sprintf('persistent-volume-claim-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
      ];
    }

    return $persistent_volume_claims;
  }

  /**
   * Create test data for k8s cluster role binding.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   Test data.
   */
  protected function createClusterRoleBindingTestFormData($repeat_count) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: rbac.authorization.k8s.io/v1
kind: ClusterRoleBinding
metadata:
  name: $name
subjects:
  -
    kind: ServiceAccount
    name: tiller
    namespace: gitlab-managed-apps
roleRef:
  apiGroup: rbac.authorization.k8s.io
  kind: ClusterRole
  name: cluster-admin
EOS;

      $data[] = [
        'name' => $name,
        'post_data' => [
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random cluster roles binding.
   *
   * @return array
   *   Random cluster roles binding array.
   */
  protected function createClusterRoleBindingsRandomTestFormData() {
    $cluster_role_bindings = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $cluster_role_bindings[] = [
        'name' => sprintf('role-binding-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
      ];
    }

    return $cluster_role_bindings;
  }

  /**
   * Create test data for k8s API service.
   *
   * @param int $repeat_count
   *   Repeat count.
   *
   * @return array
   *   Test data.
   */
  protected function createApiServiceTestFormData($repeat_count) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: apiregistration.k8s.io/v1
kind: APIService
metadata:
  name: $name
spec:
  service:null
  groupPriorityMinimum: 17800
  versionPriority: 15
EOS;

      $data[] = [
        'name' => $name,
        'post_data' => [
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random API services.
   *
   * @return array
   *   Random API services array.
   */
  protected function createApiServicesRandomTestFormData() {
    $api_services = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $api_services[] = [
        'name' => sprintf('api-service-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
      ];
    }

    return $api_services;
  }

  /**
   * Create test data for k8s role binding.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createRoleBindingTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: rbac.authorization.k8s.io/v1
kind: RoleBindingList
metadata:
  name: $name
subjects:
  -
    kind: ServiceAccount
    name: tiller
    namespace: gitlab-managed-apps
roleRef:
  apiGroup: rbac.authorization.k8s.io
  kind: Role
  name: -admin
EOS;

      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random roles binding.
   *
   * @return array
   *   Random roles binding array.
   */
  protected function createRoleBindingsRandomTestFormData() {
    $role_bindings = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $role_bindings[] = [
        'name' => sprintf('role-binding-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
      ];
    }

    return $role_bindings;
  }

  /**
   * Create test data for k8s Service Account.
   *
   * @param int $repeat_count
   *   Repeat count.
   * @param string $namespace
   *   The name of namespace.
   *
   * @return array
   *   Test data.
   */
  protected function createServiceAccountTestFormData($repeat_count, $namespace) {
    $random = new Random();

    // Input Fields.
    $data = [];
    for ($i = 0; $i < $repeat_count; $i++) {
      $name = 'name-' . $random->name(8, TRUE);
      $detail = <<<EOS
apiVersion: v1
kind: ServiceAccount
metadata:
  name: $name
secrets:
  -
    name: cloud-5-testing-8-x-2-x-xurihq-service-account-token-b8vdv
EOS;

      $data[] = [
        'name' => $name,
        'post_data' => [
          'namespace' => $namespace,
          'detail[0][value]' => $detail,
        ],
      ];
    }

    return $data;
  }

  /**
   * Create random service accounts.
   *
   * @return array
   *   Random service accounts array.
   */
  protected function createServiceAccountsRandomTestFormData() {
    $service_accounts = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $service_accounts[] = [
        'name' => sprintf('service-account-random-data #%d - %s - %s', $i + 1, date('Y/m/d H:i:s'), $this->random->name(32, TRUE)),
      ];
    }

    return $service_accounts;
  }

}
