<?php

namespace Drupal\openstack\Service;

use Drupal\openstack\Entity\Instance;
use Drupal\openstack\Entity\Image;
use Drupal\openstack\Entity\KeyPair;
use Drupal\openstack\Entity\SecurityGroup;
use Drupal\openstack\Entity\Volume;
use Drupal\openstack\Entity\Snapshot;
use Drupal\openstack\Entity\NetworkInterface;

/**
 * Entity update methods for Batch API processing.
 */
class OpenStackBatchOperations {

  /**
   * The finish callback function.
   *
   * Deletes stale entities from the database.
   *
   * @param string $entity_type
   *   The entity type.
   * @param array $stale
   *   The stale entities to delete.
   * @param bool $clear
   *   TRUE to clear entities, FALSE keep them.
   */
  public static function finished($entity_type, array $stale, $clear = TRUE) {
    $entity_type_manager = \Drupal::entityTypeManager();
    if (count($stale) && $clear === TRUE) {
      $entity_type_manager->getStorage($entity_type)->delete($stale);
    }
  }

  /**
   * Update or create an instance entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $instance
   *   The instance array.
   */
  public static function updateInstance($cloud_context, array $instance) {
    /* @var \Drupal\openstack\Service\OpenStackEc2Service $openStackEc2Service */
    $openStackEc2Service = \Drupal::service('openstack.ec2');
    $openStackEc2Service->setCloudContext($cloud_context);

    $uid = 0;
    $instance_name = '';
    $security_groups = [];
    $timestamp = time();

    foreach ($instance['SecurityGroups'] ?: [] as $security_group) {
      $security_groups[] = $security_group['GroupName'];
    }

    $entity_id = $openStackEc2Service->getEntityId('openstack_cloud_instance', 'instance_id', $instance['InstanceId']);

    if (!empty($entity_id)) {
      /* @var \Drupal\openstack\Entity\Instance $entity */
      $entity = Instance::load($entity_id);
      if (!empty($instance_name)) {
        $entity->setName($instance['InstanceId']);
      }
      $entity->setAccountId($instance['reservation_ownerid']);
      $entity->setSecurityGroups($security_groups);
      $entity->setInstanceId($instance['InstanceId']);
      $entity->setInstanceType($instance['InstanceType']);
      $entity->setAvailabilityZone($instance['Placement']['AvailabilityZone']);
      $entity->setInstanceState($instance['State']['Name']);
      $entity->setPublicDns($instance['PublicDnsName']);
      $entity->setPrivateDns($instance['PrivateDnsName']);
      $entity->setPrivateIps($instance['PrivateIpAddress']);
      $entity->setRootDevice($instance['RootDeviceName']);
      $entity->setRootDeviceType($instance['RootDeviceType']);
      $entity->setImageId($instance['ImageId']);
      $entity->setAmiLaunchIndex($instance['AmiLaunchIndex']);
      $entity->setReservation($instance['reservation_id']);
      $entity->setKeyPairName($instance['KeyName']);
      $entity->setLaunchTime(strtotime($instance['LaunchTime']->__toString()));
      $uid > 0 ?: $entity->setOwnerById($uid);
    }
    else {
      $entity = Instance::create(
        [
          'cloud_context' => $cloud_context,
          'name' => !empty($instance_name) ? $instance_name : $instance['InstanceId'],
          'account_id' => $instance['reservation_ownerid'],
          'security_groups' => $security_groups,
          'instance_id' => $instance['InstanceId'],
          'instance_type' => $instance['InstanceType'],
          'availability_zone' => $instance['Placement']['AvailabilityZone'],
          'instance_state' => $instance['State']['Name'],
          'public_dns' => $instance['PublicDnsName'],
          'private_dns' => $instance['PrivateDnsName'],
          'private_ips' => $instance['PrivateIpAddress'],
          'root_device_type' => $instance['RootDeviceType'],
          'root_device' => $instance['RootDeviceName'],
          'image_id' => $instance['ImageId'],
          'ami_launch_index' => $instance['AmiLaunchIndex'],
          'reservation' => $instance['reservation_id'],
          'key_pair_name' => $instance['KeyName'],
          'launch_time' => strtotime($instance['LaunchTime']->__toString()),
          'changed' => $timestamp,
          'refreshed' => $timestamp,
          'uid' => $uid,
        ]
      );
    }
    // Store the image name.
    $images = $openStackEc2Service->describeImages(['ImageIds' => [$instance['ImageId']]]);
    foreach ($images['Images'] ?: [] as $image) {
      if ($image['ImageId'] === $instance['ImageId']) {
        $entity->setImageName($image['Name']);
      }
    }
    $entity->save();
  }

  /**
   * Update or create a image entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $image
   *   The image array.
   */
  public static function updateImage($cloud_context, array $image) {
    /* @var \Drupal\openstack\Service\OpenStackEc2Service $openStackEc2Service */
    $openStackEc2Service = \Drupal::service('openstack.ec2');
    $openStackEc2Service->setCloudContext($cloud_context);
    $timestamp = time();

    $block_devices = [];

    foreach ($image['BlockDeviceMappings'] ?? [] as $block_device) {
      $block_devices[] = $block_device['DeviceName'];
    }

    $name = $openStackEc2Service->getTagName($image, $image['Name']);
    $uid = $openStackEc2Service->getUidTagValue($image, Image::TAG_CREATED_BY_UID);
    $entity_id = $openStackEc2Service->getEntityId('openstack_cloud_image', 'image_id', $image['ImageId']);

    // Skip if $entity already exists, by updating 'refreshed' time.
    if (!empty($entity_id)) {
      try {
        $entity = Image::load($entity_id);
        $entity->setName($name);
        $entity->setRefreshed($timestamp);
        $entity->setVisibility($image['Public']);
        $entity->setStatus($image['State']);
        $uid > 0 ?: $entity->setOwnerById($uid);
        $entity->save();
      }
      catch (\Exception $e) {
        // @TODO: Add error handling.
        \Drupal::messenger()->addError("An error occurred: {$e->toString()}");
      }
    }
    else {
      $entity = Image::create([
        'cloud_context' => $cloud_context,
        'image_id' => $image['ImageId'],
        'account_id' => $image['OwnerId'],
        'architecture' => $image['Architecture'],
        'virtualization_type' => $image['VirtualizationType'] ?? '',
        'root_device_type' => $image['RootDeviceType'] ?? '',
        'root_device_name' => $image['RootDeviceName'] ?? '',
        'ami_name' => $image['Name'],
        'name' => $image['Name'],
        'kernel_id' => $image['KernelId'] ?? '',
        'ramdisk_id' => $image['RamdiskId'] ?? '',
        'image_type' => $image['ImageType'],
        'product_code' => isset($image['ProductCodes']) ? implode(',', array_column($image['ProductCodes'], 'ProductCode')) : '',
        'source' => $image['ImageLocation'],
        'state_reason' => isset($image['StateReason']) ? $image['StateReason']['Message'] : '',
        'platform' => $image['Platform'] ?? '',
        'description' => $image['Description'] ?? '',
        'visibility' => $image['Public'],
        'block_devices' => implode(', ', $block_devices),
        'status' => $image['State'],
        'created' => strtotime($image['CreationDate']),
        'changed' => $timestamp,
        'refreshed' => $timestamp,
        'uid' => $uid,
      ]);
      $entity->save();
    }
  }

  /**
   * Update or create a key pair entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $key_pair
   *   The key_pair array.
   */
  public static function updateKeyPair($cloud_context, array $key_pair) {
    /* @var \Drupal\openstack\Service\OpenStackEc2Service $openStackEc2Service */
    $openStackEc2Service = \Drupal::service('openstack.ec2');
    $openStackEc2Service->setCloudContext($cloud_context);
    $timestamp = time();

    $entity_id = $openStackEc2Service->getEntityId('openstack_cloud_key_pair', 'key_pair_name', $key_pair['KeyName']);

    if (!empty($entity_id)) {
      $entity = KeyPair::load($entity_id);
      if (!empty($entity)) {
        $entity->setRefreshed($timestamp);
        $entity->save();
      }
    }
    else {
      $entity = KeyPair::create([
        'cloud_context' => $cloud_context,
        'key_pair_name' => $key_pair['KeyName'],
        'key_fingerprint' => $key_pair['KeyFingerprint'],
        'created' => $timestamp,
        'changed' => $timestamp,
        'refreshed' => $timestamp,
      ]);
      $entity->save();
    }
  }

  /**
   * Update or create a security group entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $security_group
   *   The security_group array.
   */
  public static function updateSecurityGroup($cloud_context, array $security_group) {
    /* @var \Drupal\openstack\Service\OpenStackEc2Service $openStackEc2Service */
    $openStackEc2Service = \Drupal::service('openstack.ec2');
    $openStackEc2Service->setCloudContext($cloud_context);

    $timestamp = time();
    $name = $openStackEc2Service->getTagName($security_group, $security_group['GroupName']);
    $uid = $openStackEc2Service->getUidTagValue($security_group, SecurityGroup::TAG_CREATED_BY_UID);
    $entity_id = $openStackEc2Service->getEntityId('openstack_cloud_security_group', 'group_id', $security_group['GroupId']);

    // Skip if $entity already exists, by updating 'refreshed' time.
    if (!empty($entity_id)) {
      /* @var \Drupal\openstack\Entity\SecurityGroup $entity */
      try {
        $entity = SecurityGroup::load($entity_id);
        $entity->setName($name);
        $entity->setRefreshed($timestamp);
        $uid ?: $entity->setOwnerById($uid);
      }
      catch (\Exception $e) {
        // @TODO: Add error handling.
        \Drupal::messenger()->addError("An error occurred: {$e->toString()}");
      }
    }
    else {
      // Create a brand new SecurityGroup entity.
      $entity = SecurityGroup::create([
        'cloud_context' => $cloud_context,
        'name' => $security_group['GroupName'] ?? $security_group['GroupId'],
        'group_id' => $security_group['GroupId'],
        'group_name' => $security_group['GroupName'],
        'description' => $security_group['Description'],
        'vpc_id' => $security_group['VpcId'] ?? NULL,
        'account_id' => $security_group['OwnerId'],
        'created' => $timestamp,
        'changed' => $timestamp,
        'refreshed' => $timestamp,
        'uid' => $uid,
      ]);
    }

    if (isset($security_group['VpcId']) && !empty($security_group['VpcId'])) {
      // Check if VPC is default.  This involves another API call.
      $vpcs = $openStackEc2Service->describeVpcs([
        'VpcIds' => [$security_group['VpcId']],
      ]);
      if ($vpcs['Vpcs']) {
        $default = $vpcs['Vpcs'][0]['IsDefault'];
        $entity->setDefaultVpc($default);
      }
    }

    // Setup the Inbound permissions.
    if (isset($security_group['IpPermissions'])) {
      $openStackEc2Service->setupIpPermissions($entity, 'ip_permission', $security_group['IpPermissions']);
    }

    // Setup outbound permissions.
    if (isset($security_group['VpcId']) && isset($security_group['IpPermissionsEgress'])) {
      $openStackEc2Service->setupIpPermissions($entity, 'outbound_permission', $security_group['IpPermissionsEgress']);
    }
    $entity->save();
  }

  /**
   * Update or create a volume entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $volume
   *   The volume array.
   * @param array $snapshot_id_name_map
   *   The snapshot map.
   */
  public static function updateVolume($cloud_context, array $volume, array $snapshot_id_name_map) {
    /* @var \Drupal\openstack\Service\OpenStackEc2Service $openStackEc2Service */
    $openStackEc2Service = \Drupal::service('openstack.ec2');
    $openStackEc2Service->setCloudContext($cloud_context);

    $timestamp = time();

    $attachments = [];
    foreach ($volume['Attachments'] ?: [] as $attachment) {
      $attachments[] = $attachment['InstanceId'];
    }

    $name = $openStackEc2Service->getTagName($volume, $volume['VolumeId']);
    $entity_id = $openStackEc2Service->getEntityId('openstack_cloud_volume', 'volume_id', $volume['VolumeId']);
    $uid = $openStackEc2Service->getUidTagValue($volume, Volume::TAG_CREATED_BY_UID);

    if ($uid === 0) {
      // Inherit the volume uid from the instance that launched it.
      if (count($attachments)) {
        $uid = $openStackEc2Service->getInstanceUid($attachments[0]);
      }
    }

    // Skip if $entity already exists, by updating 'refreshed' time.
    if (!empty($entity_id)) {
      /* @var \Drupal\openstack\Entity\Volume $entity */
      $entity = Volume::load($entity_id);
      $entity->setName($name);
      $entity->setRefreshed($timestamp);
      $entity->setState($volume['State']);
      $entity->setAttachmentInformation(implode(', ', $attachments));
      $entity->setCreated(strtotime($volume['CreateTime']));
      $entity->setSnapshotId($volume['SnapshotId']);
      $entity->setSnapshotName(empty($volume['SnapshotId'])
        ? ''
        : $snapshot_id_name_map[$volume['SnapshotId']]);
      $entity->setSize($volume['Size']);
      $entity->setVolumeType($volume['VolumeType']);

      $uid > 0 ?: $entity->setOwnerById($uid);
      $entity->save();
    }
    else {
      $entity = Volume::create([
        'cloud_context' => $cloud_context,
        'name' => $name,
        'volume_id' => $volume['VolumeId'],
        'size' => $volume['Size'],
        'state' => $volume['State'],
        'volume_status' => $volume['VirtualizationType'] ?? NULL,
        'attachment_information' => implode(', ', $attachments),
        'volume_type' => $volume['VolumeType'],
        'snapshot_id' => $volume['SnapshotId'],
        'snapshot_name' => empty($volume['SnapshotId']) ? '' : $snapshot_id_name_map[$volume['SnapshotId']],
        'availability_zone' => $volume['AvailabilityZone'],
        'encrypted' => $volume['Encrypted'],
        'kms_key_id' => $volume['KmsKeyId'] ?? NULL,
        'created' => strtotime($volume['CreateTime']),
        'changed' => $timestamp,
        'refreshed' => $timestamp,
        'uid' => $uid,
      ]);
      $entity->save();
    }
  }

  /**
   * Update or create a snapshot entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $snapshot
   *   The snapshot array.
   */
  public static function updateSnapshot($cloud_context, array $snapshot) {
    /* @var \Drupal\openstack\Service\OpenStackEc2Service $openStackEc2Service */
    $openStackEc2Service = \Drupal::service('openstack.ec2');
    $openStackEc2Service->setCloudContext($cloud_context);

    $timestamp = time();
    $name = $openStackEc2Service->getTagName($snapshot, $snapshot['SnapshotId']);
    $entity_id = $openStackEc2Service->getEntityId('openstack_cloud_snapshot', 'snapshot_id', $snapshot['SnapshotId']);
    $uid = $openStackEc2Service->getUidTagValue($snapshot, Snapshot::TAG_CREATED_BY_UID);

    if (isset($snapshot['Encrypted'])) {
      $snapshot_encrypetd = $snapshot['Encrypted'] === FALSE ? 'Not Encrypted' : 'Encrypted';
    }

    if (!empty($entity_id)) {
      $entity = Snapshot::load($entity_id);
      if (!empty($entity)) {
        $entity->setName($name);
        $entity->setStatus($snapshot['State']);
        $entity->setRefreshed($timestamp);
        $uid > 0 ?: $entity->setOwnerById($uid);
        $entity->setCreated(strtotime($snapshot['StartTime']));
        $entity->save();
      }
    }
    else {
      $entity = Snapshot::create([
        'cloud_context' => $cloud_context,
        'name' => $name,
        'snapshot_id' => $snapshot['SnapshotId'],
        'size' => $snapshot['VolumeSize'],
        'description' => $snapshot['Description'],
        'status' => $snapshot['State'],
        'volume_id' => $snapshot['VolumeId'],
        'progress' => $snapshot['Progress'],
        'encrypted' => $snapshot_encrypetd ?? NULL,
        'kms_key_id' => $snapshot['KmsKeyId'] ?? NULL,
        'account_id' => $snapshot['OwnerId'],
        'owner_aliases' => $snapshot['OwnerAlias'] ?? NULL,
        'state_message' => $snapshot['StateMessage'] ?? NULL,
        'created' => strtotime($snapshot['StartTime']),
        'changed' => $timestamp,
        'refreshed' => $timestamp,
        'uid' => $uid,
      ]);
      $entity->save();
    }
  }

  /**
   * Update or create a network interface entity.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param array $network_interface
   *   The network interface array.
   */
  public static function updateNetworkInterface($cloud_context, array $network_interface) {
    /* @var \Drupal\openstack\Service\OpenStackEc2Service $openStackEc2Service */
    $openStackEc2Service = \Drupal::service('openstack.ec2');
    $openStackEc2Service->setCloudContext($cloud_context);
    $timestamp = time();

    // Set up the primary and secondary private IP addresses.
    // Setup the allocation_ids.  The allocation_ids are used during Elastic
    // IP assignment.
    $primary_private_ip = NULL;
    $secondary_private_ip = NULL;
    $primary_association_id = NULL;
    $secondary_association_id = NULL;
    $public_ips = NULL;

    foreach ($network_interface['PrivateIpAddresses'] ?: [] as $private_ip_address) {
      if (!empty($private_ip_address['Primary'])) {
        $primary_private_ip = $private_ip_address['PrivateIpAddress'];
        if (!empty($private_ip_address['Association'])) {
          if (!empty($private_ip_address['Association']['AssociationId'])) {
            $primary_association_id = $private_ip_address['Association']['AssociationId'];
          }
          if (!empty($private_ip_address['Association']['PublicIp'])) {
            $public_ips[] = $private_ip_address['Association']['PublicIp'];
          }
        }
      }
      else {
        $secondary_private_ip = $private_ip_address['PrivateIpAddress'];
        if (!empty($private_ip_address['Association'])) {
          if (!empty($private_ip_address['Association']['AssociationId'])) {
            $secondary_association_id = $private_ip_address['Association']['AssociationId'];
          }
          if (!empty($private_ip_address['Association']['PublicIp'])) {
            $public_ips[] = $private_ip_address['Association']['PublicIp'];
          }
        }
      }
    }

    $security_groups = [];
    foreach ($network_interface['Groups'] ?: [] as $security_group) {
      $security_groups[] = $security_group['GroupName'];
    }

    // The tag key of the network interface is 'TagSet'.
    // So changing the key to align to other entities.
    // If this key changes to 'Tags' on OpenStack API, this block needs to be
    // deleted.
    if (!empty($network_interface['TagSet'])) {
      $network_interface['Tags'] = $network_interface['TagSet'];
    }

    $name = $openStackEc2Service->getTagName($network_interface, $network_interface['NetworkInterfaceId']);
    $uid = $openStackEc2Service->getUidTagValue($network_interface, NetworkInterface::TAG_CREATED_BY_UID);
    $entity_id = $openStackEc2Service->getEntityId('openstack_network_interface', 'network_interface_id', $network_interface['NetworkInterfaceId']);

    $allocation_id = NULL;
    if (!empty($network_interface['Association'])) {
      $allocation_id = $network_interface['Association']['AllocationId'] ?? NULL;
    }

    $attachment_id = NULL;
    $attachment_owner = NULL;
    $attachment_status = NULL;
    $instance_id = NULL;
    $device_index = NULL;
    $delete_on_termination = NULL;
    if (!empty($network_interface['Attachment'])) {
      $attachment_id = $network_interface['Attachment']['AttachmentId'] ?? NULL;
      $attachment_owner = $network_interface['Attachment']['InstanceOwnerId'] ?? NULL;
      $attachment_status = $network_interface['Attachment']['Status'] ?? NULL;
      $instance_id = $network_interface['Attachment']['InstanceId'] ?? NULL;
      $device_index = $network_interface['Attachment']['DeviceIndex'] ?? NULL;
      $delete_on_termination = $network_interface['Attachment']['DeleteOnTermination'] ?? NULL;
    }

    $network_vpc_id = $network_interface['VpcId'] ?? NULL;

    // Skip if $entity already exists, by updating 'refreshed' time.
    if (!empty($entity_id)) {
      /* @var \Drupal\openstack\Entity\NetworkInterface $entity */
      $entity = NetworkInterface::load($entity_id);
      $entity->setName($name);
      $entity->setRefreshed($timestamp);
      $entity->setPrimaryPrivateIp($primary_private_ip);
      $entity->setSecondaryPrivateIp($secondary_private_ip);
      $entity->setAssociationId($primary_association_id);
      $entity->setSecondaryAssociationId($secondary_association_id);
      $entity->setAttachmentId($attachment_id);
      $entity->setAttachmentOwner($attachment_owner);
      $entity->setAttachmentStatus($attachment_status);
      $entity->setSecurityGroups($security_groups);
      if (!empty($public_ips)) {
        $public_ips = implode(', ', $public_ips);
      }

      $entity->setPublicIps($public_ips);
      $entity->setVpcId($network_vpc_id);
      $entity->setStatus($network_interface['Status']);
      $uid > 0 ?: $entity->setOwnerById($uid);
      $entity->save();
    }
    else {
      $entity = NetworkInterface::create([
        'cloud_context' => $cloud_context,
        'name' => $network_interface['NetworkInterfaceId'],
        'network_interface_id' => $network_interface['NetworkInterfaceId'],
        'vpc_id' => $network_vpc_id,
        'mac_address' => $network_interface['MacAddress'],
        'security_groups' => implode(', ', $security_groups),
        'status' => $network_interface['Status'],
        'private_dns' => $network_interface['PrivateDnsName'] ?? NULL,
        'primary_private_ip' => $primary_private_ip,
        'secondary_private_ips' => $secondary_private_ip,
        'attachment_id' => $attachment_id,
        'attachment_owner' => $attachment_owner,
        'attachment_status' => $attachment_status,
        'account_id' => $network_interface['OwnerId'],
        'association_id' => $primary_association_id,
        'secondary_association_id' => $secondary_association_id,
        'subnet_id' => $network_interface['SubnetId'] ?? NULL,
        'description' => $network_interface['Description'],
        'public_ips' => $public_ips,
        'source_dest_check' => $network_interface['SourceDestCheck'] ?? NULL,
        'instance_id' => $instance_id,
        'device_index' => $device_index,
        'delete_on_termination' => $delete_on_termination,
        'allocation_id' => $allocation_id,
        'created' => $timestamp,
        'changed' => $timestamp,
        'refreshed' => $timestamp,
        'uid' => $uid,
      ]);
      $entity->save();
    }
  }

}
