<?php

namespace Drupal\cloud\EventSubscriber;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Link;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Url;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Event subscriber class for cloud module.
 */
class CloudSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * The entity type manager instance.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  private $messenger;

  /**
   * The route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  private $routeMatch;

  /**
   * Constructs a new Ec2Service object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   An entity type manager instance.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager,
                              MessengerInterface $messenger,
                              TranslationInterface $string_translation,
                              RouteMatchInterface $route_match) {

    $this->entityTypeManager = $entity_type_manager;
    $this->messenger = $messenger;

    // Setup the $this->t().
    $this->stringTranslation = $string_translation;

    $this->routeMatch = $route_match;
  }

  /**
   * Redirect if there is no cloud service provider (CloudConfig).
   *
   * @param \Symfony\Component\HttpKernel\Event\FilterResponseEvent $event
   *   The filter response event.
   */
  public function redirectIfEmpty(FilterResponseEvent $event) {
    $route_names = [
      'view.cloud_config.list',
      'view.cloud_server_template.list',
      'entity.cloud_server_template.collection',
    ];

    try {

      $ids = $this->entityTypeManager
        ->getStorage('cloud_config')
        ->loadMultiple();

      $cloud_config_type = $this->entityTypeManager
        ->getStorage('cloud_config_type')
        ->loadMultiple();

    }
    catch (\Exception $e) {
      // Cloud module is solely enabled w/o any cloud service provider module(s)
      // such as aws_cloud, k8s and/or openstack since there are no cloud_config
      // or cloud_config_type entities.
      $this->entityTypeManager->clearCachedDefinitions();
      return;
    }

    $module_list_url = Url::fromRoute('system.modules_list');

    if (in_array($this->routeMatch->getRouteName(), $route_names)) {
      // Return if not a master request.
      if (!$event->isMasterRequest()) {
        return;
      }

      // Return if not 200.
      $response = $event->getResponse();
      if (!empty($response) && $response->getStatusCode() !== 200) {
        return;
      }

      if (empty($ids)) {
        $url = Url::fromRoute('entity.cloud_config.add_page');
        $page_link = Link::fromTextAndUrl($this->t('cloud service provider'), $url)->toString();
        $this->messenger->addMessage(
          $this->t('There is no cloud service provider. Please create a new @link.', [
            '@link' => $page_link,
          ])
        );
        $response = new RedirectResponse($url->toString());
        $event->setResponse($response);
      }

      // Redirect to module list page
      // when cloud service provider type is not exist.
      if (empty($ids) && empty($cloud_config_type)) {
        $response = new RedirectResponse($module_list_url->toString());
        $event->setResponse($response);
      }
    }

    if (empty($ids) && empty($cloud_config_type)) {
      $module_page_link = Link::fromTextAndUrl($this->t('AWS Cloud, Kubernetes and/or OpenStack'), $module_list_url)->toString();
      $this->messenger->addWarning(
        $this->t('There are no Cloud Service Provider modules enabled. Please enable @link.',
        [
          '@link' => $module_page_link,
        ])
      );
    }
  }

  /**
   * Get Subscribed events.
   *
   * @return string[]
   *   An array of subscribed events.
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::RESPONSE][] = ['redirectIfEmpty'];
    return $events;
  }

}
